const autoPrefixBrowserList = ['last 2 version', 'safari 5', 'ie 8', 'ie 9', 'opera 12.1', 'ios 6', 'android 4'];

const gulp = require('gulp');
const concat = require('gulp-concat');
const uglify = require('gulp-uglify');
const sass = require('gulp-sass');
const minifyCSS = require('gulp-clean-css');
const autoprefixer = require('gulp-autoprefixer');
const plumber = require('gulp-plumber');
const babel = require('gulp-babel');
const rename = require('gulp-rename');

gulp.task('watch', () => gulp.watch('src/**/*.scss', gulp.series('styles')));

gulp.task(
    'styles',
    () => gulp
        .src(
            [
                './src/admin/scss/app.scss',
                './src/front/scss/app.scss',
                './src/front/scss/app-full.scss'
            ],
            {base: '.'}
        )
        .pipe(plumber({
            errorHandler: (err) => {
                console.log(err);
                this.emit('end');
            }
        }))
        .pipe(sass({errLogToConsole: true}))
        .pipe(autoprefixer({
            browsers: autoPrefixBrowserList,
            cascade: true
        }))
        .pipe(gulp.dest('.'))
);

gulp.task(
    'assets-copy',
    () => gulp
        .src(
            [
                './src/admin/**/*',
                './src/front/**/*',
                '!./**/scss/',
                '!./**/*.scss'
            ],
            {
                base: '.',
                nodir: true
            }
        )
        .pipe(rename(function (path) {
            path.dirname = path.dirname.replace('src', 'assets');
            path.dirname = path.dirname.replace('scss', 'css');
            path.extname = path.extname === '.js' || path.extname === '.css' ? '.min' + path.extname : path.extname;
        }))
        .pipe(gulp.dest('.'))
);

gulp.task(
    'styles-compress',
    () => gulp
        .src(['./assets/**/css/*.css'], {base: '.'})
        .pipe(minifyCSS({compatibility: 'ie8'}))
        .pipe(gulp.dest('.'))
);

gulp.task(
    'scripts-compress',
    () => gulp
        .src(['./assets/**/js/*.js'], {base: '.'})
        .pipe(babel({presets: ['babel-preset-es2015-script']}))
        .pipe(uglify())
        .pipe(gulp.dest('.'))
);

gulp.task(
    'scripts-concat',
    () => gulp
        .src([
            './src/front/js/jquery.serializejson.js',
            './src/front/js/wNumb.js',
            './src/front/js/nouislider.js',
            './src/front/js/nouislider-launcher.js',
            './src/front/js/app.js',
            './src/front/js/hooks.js'
        ])
        .pipe(concat('scripts.min.js'))
        .pipe(babel({presets: ['babel-preset-es2015-script']}))
        .pipe(uglify())
        .pipe(gulp.dest('./assets/front/js/'))
);

gulp.task(
    'default',
    gulp.series(
        'styles',
        'assets-copy',
        gulp.parallel([
            'scripts-compress',
            'scripts-concat',
            'styles-compress'
        ])
    )
);
