<?php
namespace WCStepFilter;

/**
 * Filter Class
 *
 * @class Filter
 * @version 4.3.0
 */
class Filter
{
    // <editor-fold desc="Properties">
    /**
     * Settings model variable
     *
     * @var array
     */
    public static $settingsModel;

    /**
     * Active steps session keys variable
     *
     * @var string
     */
    public static $activeStepsSessionKey = 'wcsf-active-step';

    /**
     * Filter value session keys variable
     *
     * @var string
     */
    public static $valuesSessionKey = 'wcsf-values';
    // </editor-fold>

    // <editor-fold desc="Core">
    /** Constructor */
    public function __construct()
    {
        add_action('init', [$this, 'onInit']);

        // post fields
        add_action('add_meta_boxes', [$this, 'addMetaBoxes']);
        add_action('save_post_wcsf', [$this, 'savePostAction']);

        // ajax
        add_action('wp_ajax_nopriv_WCSFGetStep', [$this, 'getStepHtmlAjax']);
        add_action('wp_ajax_WCSFGetStep', [$this, 'getStepHtmlAjax']);
        add_action('wp_ajax_nopriv_WCSFSubmit', [$this, 'submitAjax']);
        add_action('wp_ajax_WCSFSubmit', [$this, 'submitAjax']);
        add_action('wp_ajax_nopriv_WCSFSubmitAndGoNext', [$this, 'submitAndGoNextAjax']);
        add_action('wp_ajax_WCSFSubmitAndGoNext', [$this, 'submitAndGoNextAjax']);
        add_action('wp_ajax_nopriv_WCSFReset', [$this, 'resetAjax']);
        add_action('wp_ajax_WCSFReset', [$this, 'resetAjax']);
        add_action('wp_ajax_nopriv_WCSFGetPreliminaryResultsCount', [$this, 'getPreliminaryResultsCountAjax']);
        add_action('wp_ajax_WCSFGetPreliminaryResultsCount', [$this, 'getPreliminaryResultsCountAjax']);
    }

    /** On init action */
    public function onInit()
    {
        $defaultResultsURL = '/';

        if (function_exists('wc_get_page_id')) {
            $defaultResultsURL = str_replace(home_url(), '', get_permalink(wc_get_page_id('shop')));
        }

        self::$settingsModel = apply_filters(
            'wcsf_settings_model',
            [
                'questions' => [
                    'label' => esc_html__('Select questions for using', 'woocommerce-step-filter'),
                    'key' => '_questions',
                    'type' => 'multi-select',
                    'default' => [],
                    'values' => Question::getPostsIdsList(),
                    'group' => esc_html__('Basic', 'woocommerce-step-filter')
                ],
                'mode' => [
                    'label' => esc_html__('Mode', 'woocommerce-step-filter'),
                    'key' => '_mode',
                    'type' => 'select',
                    'default' => 'free-walk',
                    'values' => [
                        'free-walk' => esc_html__('Free walk', 'woocommerce-step-filter'),
                        'step-by-step' => esc_html__('Step by step', 'woocommerce-step-filter'),
                        'single-step' => esc_html__('Single step', 'woocommerce-step-filter'),
                        'sequence' => esc_html__('Sequence', 'woocommerce-step-filter'),
                    ],
                    'group' => esc_html__('Basic', 'woocommerce-step-filter')
                ],
                'show_results_within' => [
                    'label' => esc_html__('Show results within the filter', 'woocommerce-step-filter'),
                    'key' => '_show_results_within',
                    'type' => 'checkbox',
                    'default' => false,
                    'group' => esc_html__('Results', 'woocommerce-step-filter')
                ],
                'results_tab_title' => [
                    'label' => esc_html__('Results tab title', 'woocommerce-step-filter'),
                    'key' => '_results_tab_title',
                    'default' => esc_html__('Results', 'woocommerce-step-filter'),
                    'type' => 'text',
                    'group' => esc_html__('Results', 'woocommerce-step-filter')
                ],
                'show_preliminary_results_count' => [
                    'label' => esc_html__('Show preliminary results count', 'woocommerce-step-filter'),
                    'key' => '_show_preliminary_results_count',
                    'type' => 'checkbox',
                    'default' => false,
                    'public' => true,
                    'group' => esc_html__('Results', 'woocommerce-step-filter')
                ],
                'preliminary_results_count_text' => [
                    'label' => esc_html__('Preliminary results count text', 'woocommerce-step-filter'),
                    'key' => '_preliminary_results_count_text',
                    'default' => esc_html__('Products found: %d', 'woocommerce-step-filter'),
                    'description' => esc_html__('%d - products count variable', 'woocommerce-step-filter'),
                    'type' => 'text',
                    'group' => esc_html__('Results', 'woocommerce-step-filter')
                ],
                'results_url' => [
                    'label' => esc_html__('Results URL', 'woocommerce-step-filter'),
                    'key' => '_results_url',
                    'type' => 'text',
                    'default' => $defaultResultsURL,
                    'description' => esc_html__(
                        'Set # to stay on the same page (for example, if you are using the filter as a widget)',
                        'woocommerce-step-filter'
                    ),
                    'group' => esc_html__('Results', 'woocommerce-step-filter')
                ],
                'max_results_number' => [
                    'label' => esc_html__('Maximum products number in results', 'woocommerce-step-filter'),
                    'key' => '_max_results_number',
                    'type' => 'number',
                    'min' => 0,
                    'default' => 0,
                    'description' => esc_html__('Zero is equal infinity', 'woocommerce-step-filter'),
                    'public' => true,
                    'group' => esc_html__('Results', 'woocommerce-step-filter')
                ],
                'collect_inquiries' => [
                    'label' => esc_html__('Collect inquiries for statistic', 'woocommerce-step-filter'),
                    'key' => '_collect_inquiries',
                    'type' => 'checkbox',
                    'default' => false,
                    'group' => esc_html__('Behavior', 'woocommerce-step-filter')
                ],
                'scrolling_top_on_update' => [
                    'label' => esc_html__('Scrolling to top on the form update', 'woocommerce-step-filter'),
                    'key' => '_scrolling_top_on_update',
                    'type' => 'checkbox',
                    'default' => true,
                    'public' => true,
                    'group' => esc_html__('Behavior', 'woocommerce-step-filter')
                ],
                'scrolling_up_gap' => [
                    'label' => esc_html__('The gap on scrolling up (px)', 'woocommerce-step-filter'),
                    'key' => '_scrolling_up_gap',
                    'type' => 'number',
                    'default' => 0,
                    'public' => true,
                    'group' => esc_html__('Behavior', 'woocommerce-step-filter')
                ],
                'nav_action' => [
                    'label' => esc_html__('Nav action', 'woocommerce-step-filter'),
                    'key' => '_nav_action',
                    'type' => 'select',
                    'default' => 'submit-and-go-to',
                    'values' => [
                        'submit-and-go-to' => esc_html__('Submit and go to step', 'woocommerce-step-filter'),
                        'go-to' => esc_html__('Go to step', 'woocommerce-step-filter')
                    ],
                    'group' => esc_html__('Behavior', 'woocommerce-step-filter')
                ],
                'nav_view' => [
                    'label' => esc_html__('Nav view', 'woocommerce-step-filter'),
                    'key' => '_nav_view',
                    'type' => 'select',
                    'default' => 'buttons',
                    'values' => [
                        'buttons' => esc_html__('Buttons', 'woocommerce-step-filter'),
                        'pills' => esc_html__('Pills', 'woocommerce-step-filter'),
                        'tabs' => esc_html__('Tabs', 'woocommerce-step-filter'),
                        'images' => esc_html__('Images', 'woocommerce-step-filter')
                    ],
                    'group' => esc_html__('View', 'woocommerce-step-filter')
                ],
                'enable_description_tab' => [
                    'label' => esc_html__('Enable description tab', 'woocommerce-step-filter'),
                    'key' => '_enable_description_tab',
                    'default' => true,
                    'type' => 'checkbox',
                    'group' => esc_html__('View', 'woocommerce-step-filter')
                ],
                'description_tab_title' => [
                    'label' => esc_html__('Description tab title', 'woocommerce-step-filter'),
                    'key' => '_description_tab_title',
                    'default' => esc_html__('Welcome', 'woocommerce-step-filter'),
                    'type' => 'text',
                    'group' => esc_html__('View', 'woocommerce-step-filter')
                ],
                'enable_back_button' => [
                    'label' => esc_html__('Enable "Back" button', 'woocommerce-step-filter'),
                    'key' => '_enable_back_button',
                    'default' => true,
                    'type' => 'checkbox',
                    'group' => esc_html__('Controls', 'woocommerce-step-filter')
                ],
                'back_button_text' => [
                    'label' => esc_html__('"Back" button text', 'woocommerce-step-filter'),
                    'key' => '_back_button_text',
                    'default' => esc_html__('Back', 'woocommerce-step-filter'),
                    'type' => 'text',
                    'group' => esc_html__('Controls', 'woocommerce-step-filter')
                ],
                'enable_skip_button' => [
                    'label' => esc_html__('Enable "Skip" button', 'woocommerce-step-filter'),
                    'key' => '_enable_skip_button',
                    'default' => true,
                    'type' => 'checkbox',
                    'group' => esc_html__('Controls', 'woocommerce-step-filter')
                ],
                'skip_button_text' => [
                    'label' => esc_html__('"Skip" button text', 'woocommerce-step-filter'),
                    'key' => '_skip_button_text',
                    'default' => esc_html__('Skip', 'woocommerce-step-filter'),
                    'type' => 'text',
                    'group' => esc_html__('Controls', 'woocommerce-step-filter')
                ],
                'show_next_button' => [
                    'label' => esc_html__('Show "Next" button', 'woocommerce-step-filter'),
                    'key' => '_show_next_button',
                    'default' => 'until-last-step',
                    'type' => 'select',
                    'values' => [
                        'until-last-step' => esc_attr__('Until the last step', 'woocommerce-step-filter'),
                        'always' => esc_attr__('Always', 'woocommerce-step-filter'),
                        'never' => esc_attr__('Never', 'woocommerce-step-filter')
                    ],
                    'group' => esc_html__('Controls', 'woocommerce-step-filter')
                ],
                'next_button_text' => [
                    'label' => esc_html__('"Next" button text', 'woocommerce-step-filter'),
                    'key' => '_next_button_text',
                    'default' => esc_html__('Next', 'woocommerce-step-filter'),
                    'type' => 'text',
                    'group' => esc_html__('Controls', 'woocommerce-step-filter')
                ],
                'enable_reset_button' => [
                    'label' => esc_html__('Enable "Reset" button', 'woocommerce-step-filter'),
                    'key' => '_enable_reset_button',
                    'default' => true,
                    'type' => 'checkbox',
                    'group' => esc_html__('Controls', 'woocommerce-step-filter')
                ],
                'reset_button_text' => [
                    'label' => esc_html__('"Reset" button text', 'woocommerce-step-filter'),
                    'key' => '_reset_button_text',
                    'default' => esc_html__('Reset', 'woocommerce-step-filter'),
                    'type' => 'text',
                    'group' => esc_html__('Controls', 'woocommerce-step-filter')
                ],
                'show_results_button' => [
                    'label' => esc_html__('Show "Results" button', 'woocommerce-step-filter'),
                    'key' => '_show_results_button',
                    'default' => 'on-answer',
                    'type' => 'select',
                    'values' => [
                        'on-answer' => esc_attr__('Then have an answer', 'woocommerce-step-filter'),
                        'last-step' => esc_attr__('On the last step', 'woocommerce-step-filter'),
                        'always' => esc_attr__('Always', 'woocommerce-step-filter'),
                        'never' => esc_attr__('Never', 'woocommerce-step-filter')
                    ],
                    'group' => esc_html__('Controls', 'woocommerce-step-filter')
                ],
                'results_button_text' => [
                    'label' => esc_html__('"Results" button text', 'woocommerce-step-filter'),
                    'key' => '_results_button_text',
                    'default' => esc_html__('Show results', 'woocommerce-step-filter'),
                    'type' => 'text',
                    'group' => esc_html__('Controls', 'woocommerce-step-filter')
                ],
                'enable_shop_reset_button' => [
                    'label' => esc_html__('Enable shop "Reset" button', 'woocommerce-step-filter'),
                    'key' => '_enable_shop_reset_button',
                    'default' => true,
                    'type' => 'checkbox',
                    'group' => esc_html__('Controls', 'woocommerce-step-filter')
                ],
                'shop_reset_button_text' => [
                    'label' => esc_html__('Shop "Reset" button text', 'woocommerce-step-filter'),
                    'key' => '_shop_reset_button_text',
                    'default' => esc_html__('Reset', 'woocommerce-step-filter'),
                    'type' => 'text',
                    'group' => esc_html__('Controls', 'woocommerce-step-filter')
                ],
                'enable_shop_back_button' => [
                    'label' => esc_html__('Enable shop "Back" button', 'woocommerce-step-filter'),
                    'key' => '_enable_shop_back_button',
                    'default' => true,
                    'type' => 'checkbox',
                    'group' => esc_html__('Controls', 'woocommerce-step-filter')
                ],
                'shop_back_button_text' => [
                    'label' => esc_html__('Shop "Back" button text', 'woocommerce-step-filter'),
                    'key' => '_shop_back_button_text',
                    'default' => esc_html__('Back', 'woocommerce-step-filter'),
                    'type' => 'text',
                    'group' => esc_html__('Controls', 'woocommerce-step-filter')
                ],
                'shop_back_button_url' => [
                    'label' => esc_html__('Shop "Back" button URL', 'woocommerce-step-filter'),
                    'key' => '_shop_back_button_url',
                    'default' => '',
                    'type' => 'text',
                    'description' =>
                        esc_html__('Keep blank to get previous page automatically', 'woocommerce-step-filter'),
                    'group' => esc_html__('Controls', 'woocommerce-step-filter')
                ],
                'enable_shop_repeat_button' => [
                    'label' => esc_html__('Enable shop "Back" button', 'woocommerce-step-filter'),
                    'key' => '_enable_shop_repeat_button',
                    'default' => true,
                    'type' => 'checkbox',
                    'group' => esc_html__('Controls', 'woocommerce-step-filter')
                ],
                'shop_repeat_button_text' => [
                    'label' => esc_html__('Shop "Repeat" button text', 'woocommerce-step-filter'),
                    'key' => '_shop_repeat_button_text',
                    'default' => esc_html__('Repeat', 'woocommerce-step-filter'),
                    'type' => 'text',
                    'group' => esc_html__('Controls', 'woocommerce-step-filter')
                ],
                'shop_repeat_button_url' => [
                    'label' => esc_html__('Shop "Repeat" button URL', 'woocommerce-step-filter'),
                    'key' => '_shop_repeat_button_url',
                    'default' => '',
                    'type' => 'text',
                    'description' =>
                        esc_html__('Keep blank to get previous page automatically', 'woocommerce-step-filter'),
                    'group' => esc_html__('Controls', 'woocommerce-step-filter')
                ]
            ]
        );

        $name = 'Step Filter';
        $args = [
            'label' => $name,
            'labels' => [
                'name' => $name,
                'singular_name' => $name,
                'menu_name' => $name
            ],
            'description' => esc_html__(
                'This is where you can add new step filter items',
                'woocommerce-step-filter'
            ),
            'public' => false,
            'show_ui' => true,
            'map_meta_cap' => true,
            'publicly_queryable' => false,
            'exclude_from_search' => true,
            'show_in_menu' => current_user_can('manage_woocommerce') ? 'woocommerce' : true,
            'hierarchical' => false,
            'rewrite' => false,
            'query_var' => false,
            'supports' => [
                'title',
                'thumbnail',
                'editor'
            ],
            'show_in_nav_menus' => false,
            'show_in_admin_bar' => true
        ];

        register_post_type('wcsf', $args);
    }
    // </editor-fold>

    // <editor-fold desc="Admin">
    /**
     * Save meta values
     *
     * @param integer $postId
     */
    public function savePostAction($postId)
    {
        if ((defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) || !current_user_can('edit_page', $postId)) {
            return;
        }

        foreach (self::$settingsModel as $setting) {
            $value = '';

            if (isset($_POST[$setting['key']])) {
                $value = $_POST[$setting['key']];
            }

            update_post_meta($postId, $setting['key'], $value);
        }
    }

    /** Add meta-boxes */
    public function addMetaBoxes()
    {
        add_meta_box(
            'woocommerce-step-filter',
            esc_html__('Settings', 'woocommerce-step-filter'),
            [$this, 'metaBoxCallback'],
            'wcsf',
            'normal'
        );
    }

    /**
     * Post page meta-box content
     *
     * @param object $post
     */
    public function metaBoxCallback($post)
    {
        $postSettingsModelGroups = [];
        $shortCodes = [
            'shortcode' => [
                'group' => esc_html__('Basic', 'woocommerce-step-filter'),
                'label' => esc_html__('Shortcode for using', 'woocommerce-step-filter'),
                'key' => '_shortcode',
                'type' => 'text',
                'readonly' => true,
                'default' => '[woocommerce-step-filter id="' . $post->ID . '"]'
            ]
        ];

        self::$settingsModel = $shortCodes + self::$settingsModel;

        foreach (self::$settingsModel as $settingKey => $setting) {
            $postSettingsModelGroups[$setting['group']][$settingKey] = $setting;
        }
        ?>
        <section class="wcsf-settings-group" data-component="wcsf-settings-group">
            <?php foreach ($postSettingsModelGroups as $group => $settingsModel) { ?>
                <fieldset>
                    <button class="button button-large wcsf-settings-group-toggle"
                        data-component="wcsf-settings-group-toggle"
                        data-id="<?php echo esc_attr($group); ?>"><?php echo wp_kses_post($group); ?></button>
                    <table class="form-table wcsf-settings-table wcsf-settings-group-content<?php
                        echo reset($postSettingsModelGroups) == $settingsModel ? ' is-visible' : '';
                        ?>"
                        data-component="wcsf-settings-group-content"
                        data-id="<?php echo esc_attr($group); ?>">
                        <?php foreach ($settingsModel as $modelItemKey => $modelItem) { ?>
                            <tr data-component="wcsf-setting" data-key="<?php echo esc_attr($modelItemKey); ?>">
                                <th>
                                    <label for="<?php echo esc_attr($modelItem['key']); ?>"><?php
                                        echo wp_kses_post($modelItem['label']);
                                        ?></label>
                                </th>
                                <td><?php
                                    Core::settingFieldView(
                                        $modelItem,
                                        ['values' => [$modelItem['key'] => self::getSetting($post->ID, $modelItemKey)]]
                                    );
                                    ?></td>
                            </tr>
                        <?php } ?>
                    </table>
                </fieldset>
            <?php } ?>
        </section>
        <?php
    }
    // </editor-fold>

    // <editor-fold desc="Settings">
    /**
     * Get one of post settings
     *
     * @param integer $id
     * @param string $setting
     *
     * @return string|boolean|array|float
     */
    public static function getSetting($id, $setting)
    {
        static $filterSettingsCache;

        if (isset($filterSettingsCache[$id], $filterSettingsCache[$id][$setting])) {
            return $filterSettingsCache[$id][$setting];
        }

        $value = isset(self::$settingsModel[$setting]['key'])
            ? get_post_meta($id, self::$settingsModel[$setting]['key'], true)
            : null;

        if ($value == null && isset(self::$settingsModel[$setting]['default'])) {
            $value = self::$settingsModel[$setting]['default'];
        }

        if (isset(self::$settingsModel[$setting]['type'])) {
            switch (self::$settingsModel[$setting]['type']) {
                case 'checkbox':
                    $value = filter_var($value, FILTER_VALIDATE_BOOLEAN);
                    break;

                case 'number':
                    $value = (float) $value;
                    break;

                case 'array':
                    $value = (array) $value;
                    break;
            }
        }

        if (!isset($filterSettingsCache[$id])) {
            $filterSettingsCache[$id] = [];
        }

        $filterSettingsCache[$id][$setting] = $value;

        return $value;
    }

    /**
     * Get post settings
     *
     * @param integer $id
     * @param array $args
     *
     * @return array
     */
    public static function getSettings($id, $args = [])
    {
        $defaults = [
            'public' => false
        ];

        $args = array_merge($defaults, $args);
        $output = [];

        foreach (self::$settingsModel as $settingModelKey => $settingModel) {
            if ($args['public'] && (!isset($settingModel['public']) || !$settingModel['public'])) {
                continue;
            }

            $output[$settingModelKey] = self::getSetting($id, $settingModelKey);
        }

        return $output;
    }
    // </editor-fold>

    // <editor-fold desc="Steps">
    /**
     * Get filter steps array
     *
     * @param integer $id
     *
     * @return array
     */
    public static function getSteps($id)
    {
        static $stepsCache = [];

        if (isset($stepsCache[$id])) {
            return apply_filters('wcsf_steps', $stepsCache[$id], $id);
        }

        $steps = Question::getList(self::getSetting($id, 'questions'));
        $enableDescriptionTab = self::getSetting($id, 'enable_description_tab');
        $showResultsWithin = self::getSetting($id, 'show_results_within');

        if ($enableDescriptionTab) {
            $steps = [
                'description' => (object) [
                    'id' => 'description',
                    'name' => self::getSetting($id, 'description_tab_title'),
                    'description' => get_post_field('post_content', $id),
                    'thumbnailId' => get_post_thumbnail_id($id)
                ]
            ] + $steps;
        }

        if ($showResultsWithin) {
            $steps += [
                'results' => (object) [
                    'id' => 'results',
                    'name' => self::getSetting($id, 'results_tab_title'),
                    'thumbnailId' => get_post_thumbnail_id($id)
                ]
            ];
        }

        $steps = self::filterStepsAccordingDependencies($id, $steps, self::getValue($id, $steps));
        $stepsCache[$id] = $steps;

        return apply_filters('wcsf_steps', $steps, $id);
    }

    /**
     * Remove steps which are not met their dependencies
     *
     * @param integer $id
     * @param array $steps
     * @param array $values
     *
     * @return array
     */
    public static function filterStepsAccordingDependencies($id, $steps, $values)
    {
        foreach ($steps as $stepKey => $step) {
            $dependencies = (array) Question::getSetting($step->id, 'dependencies');
            $met = false;
            $localMet = false;
            $previous = null;

            if (empty($dependencies) || empty(reset($dependencies)['question'])) {
                continue;
            }

            foreach ($dependencies as $dependency) {
                if (empty($dependency['question'])) {
                    continue;
                }

                if (!isset($values[$dependency['question']])) {
                    $localMet = false;
                } else {
                    $upperValue = isset($dependency['upper_value'])
                        && filter_var($dependency['upper_value'], FILTER_VALIDATE_BOOLEAN);

                    $numericValue = $upperValue
                        ? (float) end($values[$dependency['question']])
                        : (float) reset($values[$dependency['question']]);

                    switch ($dependency['operator']) {
                        case '=':
                            $localMet = in_array($dependency['value'], $values[$dependency['question']]);
                            break;

                        case '!=':
                            $localMet = !in_array($dependency['value'], $values[$dependency['question']]);
                            break;

                        case '<':
                            $localMet = $numericValue < (float) $dependency['value'];
                            break;

                        case '<=':
                            $localMet = $numericValue <= (float) $dependency['value'];
                            break;

                        case '>':
                            $localMet = $numericValue > (float) $dependency['value'];
                            break;

                        case '>=':
                            $localMet = $numericValue >= (float) $dependency['value'];
                            break;
                    }
                }

                // look at the previous value
                if (!is_null($previous)) {
                    $localMet = $localMet && $previous;
                }

                if ($dependency['relation'] == 'and' && end($dependencies) != $dependency) {
                    $previous = $localMet;
                } else {
                    $previous = null;
                    $met = $met || $localMet;
                }
            }

            if (!$met) {
                unset($steps[$stepKey]);
            }
        }

        return apply_filters('wcsf_filtered_steps', $steps, $id);
    }

    /**
     * Get filter steps ids array
     *
     * @param integer $id
     *
     * @return array
     */
    public static function getStepsIds($id)
    {
        $stepsIds = [];

        foreach (self::getSteps($id) as $step) {
            $stepsIds[] = $step->id;
        }

        return apply_filters('wcsf_questions_steps_ids', $stepsIds, $id);
    }

    /**
     * Get filter active step id
     *
     * @param integer $id
     *
     * @return integer
     */
    public static function getActiveStepId($id)
    {
        $stepsIds = self::getStepsIds($id);
        $activeStep = reset($stepsIds);

        if (isset($_SESSION[self::$activeStepsSessionKey][$id])
            && in_array($_SESSION[self::$activeStepsSessionKey][$id], $stepsIds)
        ) {
            $activeStep = $_SESSION[self::$activeStepsSessionKey][$id];
        }

        return $activeStep;
    }

    /**
     * Get filter active step
     *
     * @param integer $id
     *
     * @return object|null
     */
    public static function getActiveStep($id)
    {
        $steps = self::getSteps($id);
        $activeStepId = self::getActiveStepId($id);
        $metActiveItem = false;
        $mode = self::getSetting($id, 'mode');
        $output = null;

        foreach ($steps as $stepId => $step) {
            if ($mode == 'step-by-step' && $metActiveItem) {
                // in the step-by-step mode reset all next steps after the active
                self::resetValue($id, $stepId);
            }

            if ($stepId == $activeStepId) {
                $metActiveItem = true;
                $output = $step;
            }
        }

        return apply_filters('wcsf_active_step', $output, $id);
    }

    /**
     * Get filter the next active step
     *
     * @param integer $id
     *
     * @return integer|string
     */
    public static function getNextActiveStepId($id)
    {
        $stepsIds = self::getStepsIds($id);
        $activeStepId = self::getActiveStepId($id);
        $foundActive = false;

        foreach ($stepsIds as $stepId) {
            if ($foundActive) {
                return $stepId;

                break;
            }

            if ($stepId == $activeStepId) {
                $foundActive = true;
            }
        }

        return null;
    }

    /**
     * Get filter the prev active step
     *
     * @param integer $id
     *
     * @return integer|string
     */
    public static function getPrevActiveStepId($id)
    {
        $stepsIds = self::getStepsIds($id);
        $activeStepId = self::getActiveStepId($id);
        $prevStep = null;

        foreach ($stepsIds as $stepId) {
            if ($stepId == $activeStepId) {
                return $prevStep;
            }

            $prevStep = $stepId;
        }

        return null;
    }

    /**
     * Set active filter step to the session variable
     *
     * @param integer $id
     * @param integer|string $step
     */
    public static function setActiveStep($id, $step)
    {
        $_SESSION[self::$activeStepsSessionKey][$id] = $step;
    }

    /**
     * Get step description text
     *
     * @param integer $id
     * @param integer $stepId
     *
     * @return string
     */
    public static function getStepDescription($id, $stepId)
    {
        $steps = self::getSteps($id);
        $description = '';

        if (isset($steps[$stepId])) {
            $description = $steps[$stepId]->description;
        }

        return apply_filters('the_content', $description);
    }

    /**
     * Get filter active step description text
     *
     * @param integer $id
     *
     * @return string
     */
    public static function getActiveStepDescription($id)
    {
        return self::getStepDescription($id, self::getActiveStepId($id));
    }

    /**
     * Reset filter values and set the first step as active
     *
     * @param integer $id
     * @param integer $stepId
     */
    public static function resetState($id, $stepId = null)
    {
        self::resetValue($id, $stepId);

        if (!$stepId) {
            $stepsIds = self::getStepsIds($id);

            self::setActiveStep($id, $stepsIds[0]);
        }
    }
    // </editor-fold>

    // <editor-fold desc="Value">
    /**
     * Save the filter value
     * Value should be an array like [stepId => [value1 => 123, value2 = 345]]
     *
     * @param integer $id
     * @param array $value
     * @param integer $stepId
     */
    public static function setValue($id, $value, $stepId = null)
    {
        // the case of the single selected checkbox un-tick
        if ($stepId && !isset($value[$stepId])) {
            $_SESSION[self::$valuesSessionKey][$id][$stepId] = [];
            $bondedQuestions = Question::getSetting($stepId, 'bonded_questions');

            // pass value to bonded questions
            if (!empty($bondedQuestions)) {
                foreach (array_keys($_SESSION[self::$valuesSessionKey]) as $filterId) {
                    if ($filterId == $id) {
                        // skip the same filter
                        continue;
                    }

                    foreach ($bondedQuestions as $bondedQuestion) {
                        $_SESSION[self::$valuesSessionKey][$filterId][$bondedQuestion] = [];
                    }
                }
            }
        }

        foreach ($value as $stepId => $stepValues) {
            foreach ($stepValues as $stepValueKey => $stepValue) {
                // clear empty items, but not zero
                if ($stepValue == '') {
                    unset($stepValues[$stepValueKey]);
                }
            }

            $_SESSION[self::$valuesSessionKey][$id][$stepId] = $stepValues;
            $bondedQuestions = Question::getSetting($stepId, 'bonded_questions');

            // pass value to bonded questions
            if (!empty($bondedQuestions)) {
                foreach (array_keys($_SESSION[self::$valuesSessionKey]) as $filterId) {
                    if ($filterId == $id) {
                        // skip the same filter
                        continue;
                    }

                    foreach ($bondedQuestions as $bondedQuestion) {
                        $_SESSION[self::$valuesSessionKey][$filterId][$bondedQuestion] = $stepValues;
                    }
                }
            }
        }
    }

    /**
     * Reset the filter by step id or fully
     *
     * @param integer $id
     * @param integer $stepId
     */
    public static function resetValue($id, $stepId = null)
    {
        if ($stepId) {
            $_SESSION[self::$valuesSessionKey][$id][$stepId] = [];

            $bondedQuestions = Question::getSetting($stepId, 'bonded_questions');

            // pass value to bonded questions
            if (!empty($bondedQuestions)) {
                foreach (array_keys($_SESSION[self::$valuesSessionKey]) as $filterId) {
                    if ($filterId == $id) {
                        // skip the same filter
                        continue;
                    }

                    foreach ($bondedQuestions as $bondedQuestion) {
                        $_SESSION[self::$valuesSessionKey][$filterId][$bondedQuestion] = [];
                    }
                }
            }
        } else {
            $_SESSION[self::$valuesSessionKey][$id] = [];
        }
    }

    /**
     * Get the filter step value
     *
     * @param integer $id
     * @param array|null $steps - for avoiding of recursion
     *
     * @return array
     */
    public static function getValue($id, $steps = null)
    {
        $output = [];
        $steps = is_array($steps) ? $steps : self::getSteps($id);

        if (isset($_SESSION[self::$valuesSessionKey][$id])) {
            $output = $_SESSION[self::$valuesSessionKey][$id];
        }

        $output = self::removeHiddenStepsValues($id, $output, $steps);
        $output = self::removeBorderlineNumberValues($id, $output, $steps);

        return apply_filters('wcsf_value', array_filter($output), $id);
    }

    /**
     * Remove values of the hidden steps
     *
     * @param integer $id
     * @param array $values
     * @param array $steps
     *
     * @return array
     */
    public static function removeHiddenStepsValues($id, $values, $steps)
    {
        foreach (array_keys($values) as $key) {
            if (isset($steps[$key])) {
                continue;
            }

            unset($values[$key]);
        }

        return apply_filters('wcsf_filtered_hidden_steps_values', $values, $id);
    }

    /**
     * Remove number view values which equals to their min/max values
     *
     * @param integer $id
     * @param array $values
     * @param array $steps
     *
     * @return array
     */
    public static function removeBorderlineNumberValues($id, $values, $steps)
    {
        foreach (array_keys($values) as $key) {
            if (!isset($steps[$key])) {
                continue;
            }

            $step = $steps[$key];

            if (property_exists($step, 'viewType') && strpos($step->viewType, 'number') !== false) {
                $minMax = [];

                // get min/max values
                switch ($step->filterType) {
                    case 'price':
                        $minMax = Core::getMinMaxPrices();
                        break;

                    case 'attribute':
                        $minMax = Core::getMinMaAttributeValues($step->attribute);
                        break;

                    case 'meta':
                        $minMax = Core::getMinMaxMetaValues($step->metaKey);
                }

                switch ($step->viewType) {
                    case 'number-from':
                        if ($values[$key]['from'] == $minMax['min']) {
                            unset($values[$key]);
                        }

                        break;
                    case 'number-to':
                        if ($values[$key]['to'] == $minMax['min']) {
                            unset($values[$key]);
                        }

                        break;
                    case 'number-between':
                        if (isset($values[$key]['from']) && $values[$key]['from'] == $minMax['min']
                            && isset($values[$key]['to']) && $values[$key]['to'] == $minMax['max']
                        ) {
                            unset($values[$key]);
                        }
                }
            }
        }

        return apply_filters('wcsf_filtered_borderline_number_values', $values, $id);
    }
    
    /**
     * Get the filter step value list
     *
     * @param integer $id
     *
     * @return array
     */
    public static function getValueList($id)
    {
        $value = self::getValue($id);
        $steps = self::getSteps($id);
        $stepsIds = self::getStepsIds($id);
        $output = [];

        // sorting the values according the steps
        $value = array_replace(array_flip($stepsIds), $value);

        foreach ($value as $stepId => $valueItem) {
            if (!is_array($valueItem) || !isset($steps[$stepId])) {
                continue;
            }

            $values = [];
            $step = $steps[$stepId];
            $viewType = Question::getSetting($stepId, 'view_type');
            $anyProductValueText = Question::getSetting($stepId, 'any_product_value_text');
            $listItem = (object) [
                'id' => $stepId,
                'name' => wp_kses_post($step->name),
                'value' => null,
                'values' => []
            ];

            if (strpos($step->viewType, 'number') !== false) {
                $fromText = Question::getSetting($stepId, 'from_value_text');
                $toText = Question::getSetting($stepId, 'to_value_text');

                // get numeric values
                if (isset($valueItem['from'])
                    && ($step->viewType == 'number-from' || $step->viewType == 'number-between')
                ) {
                    $values[] = "$fromText $valueItem[from]";
                }

                if (isset($valueItem['to'])
                    && ($step->viewType == 'number-to' || $step->viewType == 'number-between')
                ) {
                    $values[] = "$toText $valueItem[to]";
                }

                $values[] = Question::getSetting($stepId, 'numeric_unit');
                $listItem->value = implode(' ', $values);
                $listItem->values = $values;
            } else {
                if ($step->filterType == 'manual') {
                    // get manual names
                    foreach ($valueItem as $valueItemPart) {
                        if (isset($step->manualValues[$valueItemPart])) {
                            $values[] = $step->manualValues[$valueItemPart]['name'];
                        }
                    }
                } else {
                    // define taxonomy
                    $taxonomy = $step->filterType == 'tag' ? 'product_tag' : 'product_cat';
                    $taxonomy = $step->filterType == 'attribute'
                        ? \wc_attribute_taxonomy_name_by_id((int) $step->attribute)
                        : $taxonomy;

                    // get terms values
                    foreach ($valueItem as $valueItemKey => $valueItemPart) {
                        // get multi-choice term name
                        if (strpos($viewType, 'multi-choice') !== false) {
                            if (in_array((int) $valueItemPart, [-1, 1, 2])) {
                                $term = get_term($valueItemKey, $taxonomy);

                                switch ((int) $valueItemPart) {
                                    case -1:
                                        $values[] = "<s class=\"wcsf-value-item-value-excluded\">$term->name</s>";
                                        break;

                                    case 1:
                                        $values[] = "<span class=\"wcsf-value-item-value-possible\">$term->name</span>";
                                        break;

                                    case 2:
                                    default:
                                        $values[] = "<b class=\"wcsf-value-item-value-required\">$term->name</b>";
                                }
                            }

                            continue;
                        }

                        // get "Doesn't matter" text
                        if ($valueItemPart == 0) {
                            $values[] = $anyProductValueText;

                            continue;
                        }

                        // get term name
                        $term = get_term($valueItemPart, $taxonomy);
                        $values[] = $term->name;
                    }
                }

                $listItem->value = implode(', ', $values);
                $listItem->values = $values;
            }

            if (!empty($listItem->value)) {
                $output[] = $listItem;
            }
        }

        return apply_filters('wcsf_value_list', $output, $id);
    }
    // </editor-fold>
    
    // <editor-fold desc="Fields">
    /**
     * Remove manual values which are not met their dependencies
     *
     * @param integer $id
     * @param array $items
     * @param array $values
     *
     * @return array
     */
    public static function filterManualValuesAccordingDependencies($id, $items, $values)
    {
        foreach ($items as $key => $item) {
            if (!isset($item['dependencies'])) {
                continue;
            }

            $dependencies = $item['dependencies'];
            $met = false;
            $localMet = false;
            $previous = null;

            if (empty($dependencies) || empty(reset($dependencies)['question'])) {
                continue;
            }

            foreach ($dependencies as $dependency) {
                if (empty($dependency['question'])) {
                    continue;
                }

                if (!isset($values[$dependency['question']])) {
                    $localMet = false;
                } else {
                    $upperValue = isset($dependency['upper_value'])
                        && filter_var($dependency['upper_value'], FILTER_VALIDATE_BOOLEAN);

                    $numericValue = $upperValue
                        ? (float) end($values[$dependency['question']])
                        : (float) reset($values[$dependency['question']]);

                    switch ($dependency['operator']) {
                        case '=':
                            $localMet = in_array($dependency['value'], $values[$dependency['question']]);
                            break;

                        case '!=':
                            $localMet = !in_array($dependency['value'], $values[$dependency['question']]);
                            break;

                        case '<':
                            $localMet = $numericValue < (float) $dependency['value'];
                            break;

                        case '<=':
                            $localMet = $numericValue <= (float) $dependency['value'];
                            break;

                        case '>':
                            $localMet = $numericValue > (float) $dependency['value'];
                            break;

                        case '>=':
                            $localMet = $numericValue >= (float) $dependency['value'];
                            break;
                    }
                }

                // look at the previous value
                if (!is_null($previous)) {
                    $localMet = $localMet && $previous;
                }

                if ($dependency['relation'] == 'and' && end($dependencies) != $dependency) {
                    $previous = $localMet;
                } else {
                    $previous = null;
                    $met = $met || $localMet;
                }
            }

            if (!$met) {
                unset($items[$key]);
            }
        }

        return apply_filters('wcsf_filtered_manual_values', $items, $id);
    }

    /**
     * Get filter step fields according the question settings
     *
     * @param integer $id
     * @param integer $stepId
     *
     * @return array
     */
    public static function getStepFields($id, $stepId)
    {
        if (!is_numeric($stepId)) {
            return [];
        }

        $output = [];
        $minMax = [];
        $products = [];
        $component = 'wcsf-field-input';
        $step = Question::get($stepId);
        $value = self::getValue($id);
        $showPreliminaryResultsCount = self::getSetting($id, 'show_preliminary_results_count');
        $actionOnClick = Question::getSetting($stepId, 'action_on_click');
        $currentValue = isset($value[$stepId]) ? $value[$stepId] : false;
        $viewsWithAnyProductValue = ['checkbox', 'radio', 'select', 'image-radio'];
        $disableUnmet = Question::getSetting($stepId, 'disable_unmet_values')
            && !in_array($step->viewType, ['multi-choice-checkbox', 'multi-choice-range']);

        if ($disableUnmet) {
            $products = get_posts(array_replace(
                [
                    'post_type' => 'product',
                    'fields' => 'ids',
                    'no_found_rows' => true,
                    'nopaging' => true
                ],
                Core::getQueryArgsByFilter($id, $value)
            ));
        }

        if ($actionOnClick == 'submit-and-go-next') {
            $component .= ' wcsf-submit-and-go-next';
        } elseif ($actionOnClick == 'submit') {
            $component .= ' wcsf-submit';
        } elseif ($showPreliminaryResultsCount) {
            $component .= ' wcsf-get-preliminary-results-count';
        }

        if (!isset($step->viewType)) {
            return apply_filters('wcsf_step_fields', $output, $id, $stepId);
        }

        // check dependencies
        if ($step->filterType == 'manual') {
            $step->manualValues = self::filterManualValuesAccordingDependencies($id, $step->manualValues, $value);
        }

        if (strpos($step->viewType, 'number') !== false) {
            // get min/max values
            switch ($step->filterType) {
                case 'price':
                    $minMax = Core::getMinMaxPrices($products);
                    break;

                case 'attribute':
                    $minMax = Core::getMinMaAttributeValues($step->attribute, $products);
                    break;

                case 'meta':
                    $minMax = Core::getMinMaxMetaValues($step->metaKey, $products);
                    break;
            }

            if ($step->viewType == 'number-from' || $step->viewType == 'number-between') {
                $output['from'] = (object) [
                    'label' => Question::getSetting($stepId, 'from_value_text'),
                    'name' => "value[$stepId][from]",
                    'unit' => Question::getSetting($stepId, 'numeric_unit'),
                    'step' => Question::getSetting($stepId, 'numeric_step'),
                    'value' => isset($currentValue['from']) ? $currentValue['from'] : $minMax['min'],
                    'min' => isset($minMax['min']) ? $minMax['min'] : '',
                    'max' => isset($minMax['max']) ? $minMax['max'] : '',
                    'component' => $component
                ];
            }

            if ($step->viewType == 'number-to' || $step->viewType == 'number-between') {
                $output['to'] = (object) [
                    'label' => Question::getSetting($stepId, 'to_value_text'),
                    'name' => "value[$stepId][to]",
                    'unit' => Question::getSetting($stepId, 'numeric_unit'),
                    'step' => Question::getSetting($stepId, 'numeric_step'),
                    'value' => isset($currentValue['to']) ? $currentValue['to'] : $minMax['max'],
                    'min' => isset($minMax['min']) ? $minMax['min'] : '',
                    'max' => isset($minMax['max']) ? $minMax['max'] : '',
                    'component' => $component
                ];
            }
        } else {
            if (in_array($step->filterType, ['tag', 'category', 'attribute'])) {
                $includeSubCategories = false;
                $exclude = Question::getSetting($stepId, 'excluded_terms');
                $include = Question::getSetting($stepId, 'included_terms');

                switch ($step->filterType) {
                    case 'tag':
                        $taxonomy = 'product_tag';
                        $thumbnailMetaKey = '';
                        break;

                    case 'attribute':
                        $taxonomy = \wc_attribute_taxonomy_name_by_id((int) $step->attribute);
                        $thumbnailMetaKey = '_wcsf_thumbnail';
                        break;

                    default:
                    case 'category':
                        $taxonomy = 'product_cat';
                        $thumbnailMetaKey = 'thumbnail_id';
                        break;
                }

                $queryArgs = [
                    'taxonomy' => $taxonomy,
                    'hide_empty' => true
                ];

                if ($exclude) {
                    $queryArgs['exclude'] = array_map('trim', explode(',', $exclude));
                }

                if ($include) {
                    $queryArgs['include'] = array_map('trim', explode(',', $include));
                }

                if ($step->filterType == 'category') {
                    $includeSubCategories = Question::getSetting($stepId, 'include_sub_categories');

                    $queryArgs['hide_empty'] = !$includeSubCategories;
                    $queryArgs['parent'] = (int) Question::getSetting($stepId, 'category_parent');
                }

                $queryArgs = apply_filters('wcsf_step_fields_query_args', $queryArgs, $id, $stepId);
                $terms = Core::getTermsTree(
                    $queryArgs,
                    [
                        'flatten' => !in_array($step->viewType, ['radio', 'checkbox']),
                        'showLevelInName' => $step->viewType == 'select',
                        'maxLevel' => $includeSubCategories ? 20 : 0
                    ]
                );

                $output = self::getStepTermsFields(
                    $terms,
                    $taxonomy,
                    $thumbnailMetaKey,
                    $currentValue,
                    $component,
                    $disableUnmet,
                    $products
                );

                if (Question::getSetting($stepId, 'any_product_value')
                    && in_array($step->viewType, $viewsWithAnyProductValue)
                ) {
                    $output[] = (object) [
                        'checked' => $currentValue ? in_array(0, $currentValue) : false,
                        'disabled' => false,
                        'name' => Question::getSetting($stepId, 'any_product_value_text'),
                        'value' => 0,
                        'thumbnail' => Question::getSetting($stepId, 'any_product_value_thumbnail'),
                        'component' => $component
                    ];
                }
            } elseif ($step->filterType == 'manual') {
                // prepare manual values
                foreach ($step->manualValues as $valueKey => $value) {
                    $enabled = !$disableUnmet;
                    $isChecked = $currentValue
                        ? in_array($valueKey, $currentValue)
                        : (isset($value['pre-checked']) ? (bool) $value['pre-checked'] : false);

                    if ($disableUnmet) {
                        switch ($value['filter-by']) {
                            case 'category':
                                // check is term disabled
                                foreach ($products as $product) {
                                    if (has_term($value['items'], 'product_cat', $product)) {
                                        $enabled = true;

                                        break;
                                    }
                                }

                                break;
                            case 'product':
                                $enabled = !empty(array_intersect($products, $value['items']));
                                break;

                            case 'any':
                                $enabled = true;
                                break;
                        }
                    }

                    $output[] = (object) [
                        'checked' => $isChecked,
                        'disabled' => !$isChecked && !$enabled,
                        'name' => $value['name'],
                        'value' => $valueKey,
                        'thumbnail' => $value['thumbnail'],
                        'component' => $component
                    ];
                }
            } elseif ($step->filterType == 'meta') {
                // get meta values
                $values = Core::getMetaValues($step->metaKey);

                foreach ($values as $productId => $value) {
                    $enabled = !$disableUnmet;
                    $isChecked = $currentValue ? in_array($value, $currentValue) : false;

                    if ($disableUnmet) {
                        $enabled = in_array($productId, $products);
                    }

                    $output[] = (object) [
                        'checked' => $isChecked,
                        'disabled' => !$isChecked && !$enabled,
                        'name' => $value,
                        'value' => $value,
                        'thumbnail' => null,
                        'component' => $component
                    ];
                }
            }
        }

        return apply_filters('wcsf_step_fields', $output, $id, $stepId);
    }

    /**
     * Get filter step fields by terms array
     *
     * @param array $terms
     * @param string $taxonomy
     * @param string $thumbnailMetaKey
     * @param array $current
     * @param string $component
     * @param bool $disableUnmet
     * @param array $products
     *
     * @return array
     */
    public static function getStepTermsFields(
        $terms,
        $taxonomy,
        $thumbnailMetaKey,
        $current,
        $component,
        $disableUnmet,
        $products
    ) {
        $output = [];

        foreach ($terms as $term) {
            $enabled = !$disableUnmet;
            $isPreChecked = (bool) get_term_meta($term->term_id, '_wcsf_pre_checked', true);
            $isChecked = $current ? in_array($term->term_id, $current) : $isPreChecked;
            $thumbnail = $thumbnailMetaKey ? get_term_meta($term->term_id, $thumbnailMetaKey, true) : '';

            if ($disableUnmet) {
                $children = get_term_children($term->term_id, $taxonomy);

                // check is term disabled
                foreach ($products as $product) {
                    if (has_term(array_merge($children, [$term->term_id]), $taxonomy, $product)) {
                        $enabled = true;

                        break;
                    }
                }
            }

            $outputItem = (object) [
                'checked' => $isChecked,
                'currentValue' => $current,
                'disabled' => !$isChecked && !$enabled,
                'name' => $term->name,
                'count' => $term->count,
                'value' => $term->term_id,
                'thumbnail' => $thumbnail,
                'component' => $component
            ];

            if (isset($term->children) && !empty($term->children)) {
                $outputItem->children = self::getStepTermsFields(
                    $term->children,
                    $taxonomy,
                    $thumbnailMetaKey,
                    $current,
                    $component,
                    $disableUnmet,
                    $products
                );
            }

            $output[] = $outputItem;
        }

        return $output;
    }

    /**
     * Get filter active step fields according the question settings
     *
     * @param integer $id
     *
     * @return array
     */
    public static function getActiveStepFields($id)
    {
        $output = self::getStepFields($id, self::getActiveStepId($id));

        return apply_filters('wcsf_active_step_fields', $output, $id);
    }
    // </editor-fold>

    // <editor-fold desc="Nav">
    /**
     * Get filter navigation items array
     *
     * @param integer $id
     *
     * @return array
     */
    public static function getNavItems($id)
    {
        $steps = self::getSteps($id);
        $activeStep = self::getActiveStepId($id);
        $nextActiveStepId = self::getNextActiveStepId($id);
        $mode = self::getSetting($id, 'mode');
        $navAction = self::getSetting($id, 'nav_action');
        $output = [];
        $metActiveItem = false;

        foreach ($steps as $step) {
            $isActiveStep = $activeStep == $step->id;
            $navItem = (object) [
                'id' => $step->id,
                'name' => $step->name,
                'class' => '',
                'disabled' => false,
                'thumbnailId' => $step->thumbnailId
            ];

            if ($mode == 'step-by-step') {
                if ($isActiveStep) {
                    $navItem->class = 'active';
                    $navItem->component = '';
                    $metActiveItem = true;
                } elseif ($nextActiveStepId == $step->id || !$metActiveItem) {
                    $navItem->component = "wcsf-$navAction";
                } else {
                    $navItem->disabled = true;
                    $navItem->component = '';
                }
            } else {
                $navItem->class = ($isActiveStep ? ' active' : '');
                $navItem->component = 'wcsf-tabs-nav-item' . (!$isActiveStep ? (" wcsf-$navAction") : '');
            }

            $output[] = $navItem;
        }

        return apply_filters('wcsf_nav_items', $output, $id);
    }

    /**
     * Get pagination items array for products loop
     *
     * @param array $args
     *
     * @return array
     */
    public static function getProductsPaginationItems($args)
    {
        $output = [];
        $defaults = [
            'page' => 1,
            'productsQuery' => []
        ];

        $args = array_replace_recursive($defaults, $args);

        if (!$args['productsQuery'] || empty($args['productsQuery'])) {
            return [];
        }

        $paginationArgs = apply_filters(
            'wcsf_products_pagination_args',
            [
                'format' => '?paged=%#%',
                'base' => '%_%',
                'total' => $args['productsQuery']->max_num_pages,
                'current' => max($args['page'], get_query_var('paged')),
                'show_all' => false,
                'end_size' => 1,
                'mid_size' => 2,
                'prev_next' => true,
                'prev_text'    => '&larr;',
                'next_text'    => '&rarr;',
                'type' => 'array'
            ],
            $args
        );

        $links = paginate_links($paginationArgs);

        foreach ((array) $links as $link) {
            // add custom classes
            $link = str_replace('page-numbers', 'page-numbers page-link', $link);

            // replace empty href
            $link = str_replace('href=""', 'href="?paged=1"', $link);
            $link = str_replace("href=''", 'href="?paged=1"', $link);

            preg_match_all('/<a[^>]+href=([\'"])(?<href>.+?)\1[^>]*>/i', $link, $result);

            if (!empty($result) && !empty($result['href'][0])) {
                $href = $result['href'][0];
                $linkParts = parse_url($href);

                parse_str($linkParts['query'], $linkPartsQuery);

                $paged = isset($linkPartsQuery['paged']) ? $linkPartsQuery['paged'] : 1;

                // add custom attributes
                $link = str_replace(
                    ' href=',
                    ' data-component="wcsf-products-pagination-link wcsf-go-to" data-step-id="results" data-page="'
                    . $paged . '" href=',
                    $link
                );
            }

            $output[] = (object) [
                'class' => strpos($link, 'current') !== false ? 'active' : '',
                'innerHtml' => $link
            ];
        }

        return apply_filters('wcsf_products_pagination_items', $output, $args);
    }

    /**
     * Get link URL for the shop "repeat" control
     *
     * @param integer $id
     * @param string $url
     *
     * @return string
     */
    public static function getShopRepeatUrl($id, $url)
    {
        $query = [];
        $parts = parse_url($url);

        // parse query args
        if (isset($parts['query'])) {
            parse_str($parts['query'], $query);
        }

        $query['wcsf-reset-state'] = $id;

        // build query string
        $parts['query'] = http_build_query($query);

        // stringify URL parts
        $output = Core::buildUrl($parts);

        return apply_filters('wcsf_shop_repeat_url', $output, $url, $parts);
    }
    // </editor-fold>
    
    // <editor-fold desc="Results">
    /**
     * Get filter results URL with query args
     *
     * @param integer $id
     * @param array $queryArgs
     *
     * @return string
     */
    public static function getResultsUrl($id, $queryArgs = [])
    {
        $url = self::getSetting($id, 'results_url');
        $query = [];

        // will stay on the same page
        if ($url == '#') {
            $url = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? 'https' : 'http')
                . "://$_SERVER[HTTP_HOST]$_SERVER[REQUEST_URI]";
        }

        // if the settings is empty
        if (!$url) {
            $url = get_permalink(wc_get_page_id('shop'));
        }

        // if URL is absolute
        if (strpos($url, home_url()) === false) {
            $url = home_url() . '/' . $url;
        }

        $parts = parse_url($url);

        // parse query args
        if (isset($parts['query'])) {
            parse_str($parts['query'], $query);
        }

        // redirect to the product archive
        if (!empty($query['page_id']) && '' === get_option('permalink_structure')
            && wc_get_page_id('shop') == $query['page_id']
        ) {
            $url = get_post_type_archive_link('product');
            $parts = parse_url($url);
            $query = [];

            // parse query args
            if (isset($parts['query'])) {
                parse_str($parts['query'], $query);
            }
        }

        // blend in required args
        $query['wcsf-results'] = $id;
        $query['wcsf-value'] = self::getValue($id);
        $query['wcsf-referer'] = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? 'https' : 'http')
            . "://$_SERVER[HTTP_HOST]$_SERVER[REQUEST_URI]";

        $query = array_replace($query, $queryArgs);

        // unset reset-value query
        if (isset($query['wcsf-reset-value'])) {
            unset($query['wcsf-reset-value']);
        }

        // woof plugin query emulate
        if (class_exists('\\WOOF')) {
            $query['swoof'] = 1;
        }

        // apply outer filters
        $query = apply_filters('wcsf_results_redirect_url_query', $query, $id);

        // build query string
        $parts['query'] = http_build_query($query);

        // stringify URL parts
        $redirectLink = Core::buildUrl($parts);

        return apply_filters('wcsf_results_redirect_url', $redirectLink, $id, $parts);
    }

    /**
     * Get preliminary products count by the current value
     *
     * @param integer $id
     * @param array $valuePart
     * @param array $args
     *
     * @return integer
     */
    public static function getPreliminaryResultsCount($id, $valuePart = [], $args = [])
    {
        $defaultQueryArgs = [
            'post_type' => 'product',
            'fields' => 'ids',
            'no_found_rows' => true,
            'nopaging' => true
        ];

        $value = self::getValue($id);
        $value = array_replace_recursive($value, $valuePart);
        $queryArgs = Core::getQueryArgsByFilter($id, $value, $args);
        $queryArgs = array_replace_recursive($defaultQueryArgs, $queryArgs);
        $query = new \WP_Query($queryArgs);

        return (int) $query->post_count;
    }

    /**
     * Get pagination items array for products loop
     *
     * @param integer $id
     * @param array $args
     * @param string $orderBy
     *
     * @return array
     */
    public static function getProductsQueryArgs($id, $args = [], $orderBy = 'menu_order')
    {
        $defaults = [
            'page' => 1
        ];

        $args = array_replace($defaults, $args);
        $orderParts = explode('-', $orderBy);
        $order = isset($orderParts[1]) ? $orderParts[1] : 'asc';
        $value = self::getValue($id);
        $output = Core::getQueryArgsByFilter($id, $value);
        $wcQuery = new \WC_Query();
        $orderingArgs = $wcQuery->get_catalog_ordering_args($orderParts[0], $order);
        $defaults = [
            'post_type' => 'product',
            'posts_per_page' => wc_get_default_products_per_row() * wc_get_default_product_rows_per_page(),
            'orderby' => $orderBy,
            'paged' => $args['page']
        ];

        $output = array_replace($defaults, $args, $output, $orderingArgs);

        return apply_filters('wcsf_products_query_args', $output, $id, $args);
    }
    // </editor-fold>

    // <editor-fold desc="Get">
    /**
     * Returns array of ids and titles of posts
     *
     * @param array $args
     *
     * @return array
     */
    public static function getPostsIdsList($args = [])
    {
        $defaults = [
            'post_type' => 'wcsf',
            'numberposts' => -1,
            'post_status' => 'publish'
        ];

        $args = array_replace($defaults, $args);
        $output = [];
        $posts = get_posts($args);

        foreach ($posts as $post) {
            $output[$post->ID] = $post->post_title;
        }

        return $output;
    }
    // </editor-fold>

    // <editor-fold desc="AJAX">
    /** Save value and get form template by ajax */
    public function submitAjax()
    {
        $id = isset($_POST['id']) ? (int) $_POST['id'] : null;
        $stepId = isset($_POST['stepId']) ? (string) $_POST['stepId'] : null;
        $value = isset($_POST['value']) && !empty($_POST['value']) ? (array) $_POST['value'] : [$stepId => []];
        $page = isset($_POST['page']) ? (int) $_POST['page'] : 1;
        $orderBy = isset($_POST['orderBy']) ? (string) $_POST['orderBy'] : 'menu_order';

        // save value
        self::setValue($id, $value, $stepId);

        wp_send_json([
            'content' => Core::getTemplatePart(
                'router',
                [
                    'id' => $id,
                    'page' => $page,
                    'orderBy' => $orderBy
                ],
                ['echo' => false]
            )
        ]);
    }

    /** Save value and get the next form template by ajax */
    public function submitAndGoNextAjax()
    {
        $id = isset($_POST['id']) ? (int) $_POST['id'] : null;
        $stepId = isset($_POST['stepId']) ? (string) $_POST['stepId'] : null;
        $value = isset($_POST['value']) && !empty($_POST['value']) ? (array) $_POST['value'] : [$stepId => []];
        $page = isset($_POST['page']) ? (int) $_POST['page'] : 1;
        $orderBy = isset($_POST['orderBy']) ? (string) $_POST['orderBy'] : 'menu_order';
        $referer = isset($_POST['referer']) ? (string) $_POST['referer'] : home_url();

        // save value
        self::setValue($id, $value, $stepId);

        $stepsIds = self::getStepsIds($id);

        if (end($stepsIds) == $stepId) {
            wp_send_json(['redirect' => self::getResultsUrl($id, ['wcsf-referer' => $referer])]);
        }

        // set active step from the step id and increment it
        self::setActiveStep($id, $stepId);
        self::setActiveStep($id, self::getNextActiveStepId($id));

        wp_send_json([
            'content' => Core::getTemplatePart(
                'router',
                [
                    'id' => $id,
                    'page' => $page,
                    'orderBy' => $orderBy
                ],
                ['echo' => false]
            )
        ]);
    }

    /** Save value and get the form template by ajax */
    public function getStepHtmlAjax()
    {
        $id = isset($_POST['id']) ? (int) $_POST['id'] : null;
        $stepId = isset($_POST['stepId']) ? (string) $_POST['stepId'] : null;
        $value = isset($_POST['value']) && !empty($_POST['value']) ? (array) $_POST['value'] : null;
        $page = isset($_POST['page']) ? (int) $_POST['page'] : 1;
        $orderBy = isset($_POST['orderBy']) ? (string) $_POST['orderBy'] : 'menu_order';

        // save value
        if ($value) {
            self::setValue($id, $value);
        }

        // change step
        if ($stepId) {
            self::setActiveStep($id, $stepId);
        }

        wp_send_json([
            'content' => Core::getTemplatePart(
                'router',
                [
                    'id' => $id,
                    'stepId' => $stepId,
                    'page' => $page,
                    'orderBy' => $orderBy
                ],
                ['echo' => false]
            )
        ]);
    }

    /** Reset filter by ajax */
    public function resetAjax()
    {
        $id = isset($_POST['id']) ? (int) $_POST['id'] : null;
        $stepId = isset($_POST['stepId']) ? (string) $_POST['stepId'] : null;

        if (!$id) {
            return;
        }

        self::resetState($id, $stepId);

        wp_send_json(['content' => Core::getTemplatePart('router', ['id' => $id], ['echo' => false])]);
    }

    /** Get preliminary results count by ajax */
    public function getPreliminaryResultsCountAjax()
    {
        $id = isset($_POST['id']) ? (int) $_POST['id'] : null;

        if (!$id) {
            return;
        }

        $value = isset($_POST['value']) && !empty($_POST['value']) ? (array) $_POST['value'] : [];
        $categoryId = isset($_POST['categoryId']) && !empty($_POST['categoryId']) ? (int) $_POST['categoryId'] : null;
        $tagId = isset($_POST['tagId']) && !empty($_POST['tagId']) ? (int) $_POST['tagId'] : null;
        $args = [];

        if ($categoryId) {
            $args['categoryId'] = $categoryId;
        }

        if ($tagId) {
            $args['tagId'] = $tagId;
        }

        $preliminaryResultsCountText = self::getSetting($id, 'preliminary_results_count_text');
        $output = self::getPreliminaryResultsCount($id, $value, $args);

        wp_send_json([
            'message' => sprintf($preliminaryResultsCountText, $output),
            'value' => $output
        ]);
    }
    // </editor-fold>
}
