<?php
namespace WCStepFilter;

/**
 * Question Class
 *
 * @class Question
 * @version 4.8.0
 */
class Question
{
    // <editor-fold desc="Properties">
    /**
     * Settings model variable
     *
     * @var array
     */
    public static $settingsModel;
    // </editor-fold>

    // <editor-fold desc="Core">
    /** Constructor */
    public function __construct()
    {
        add_action('init', [$this, 'onInit']);

        // admin scripts enqueue
        add_action('admin_enqueue_scripts', [$this, 'enqueueAdmin']);

        // post fields
        add_action('add_meta_boxes', [$this, 'addMetaBoxes']);
        add_action('save_post_wcsf_question', [$this, 'savePostAction']);

        // ajax
        add_action('wp_ajax_wcsfAddQuestionValue', [$this, 'addQuestionValueAjax']);
        add_action('wp_ajax_wcsfGetQuestionValuesSelectOptions', [$this, 'getValuesSelectOptionsAjax']);
    }

    /** On init action */
    public function onInit()
    {
        self::$settingsModel = apply_filters(
            'wcsf_question_settings_model',
            [
                'view_type' => [
                    'label' => esc_html__('View type', 'woocommerce-step-filter'),
                    'key' => '_view_type',
                    'type' => 'select',
                    'default' => 'select',
                    'values' => [
                        'radio' => esc_html__('Radio', 'woocommerce-step-filter'),
                        'select' => esc_html__('Select', 'woocommerce-step-filter'),
                        'checkbox' => esc_html__('Checkbox', 'woocommerce-step-filter'),
                        'multi-choice-checkbox' => esc_html__('Multi-choice checkbox', 'woocommerce-step-filter'),
                        'multi-choice-range' => esc_html__('Multi-choice range', 'woocommerce-step-filter'),
                        'image-radio' => esc_html__('Image radio', 'woocommerce-step-filter'),
                        'image-checkbox' => esc_html__('Image checkbox', 'woocommerce-step-filter'),
                        'number-from' => esc_html__('Number from', 'woocommerce-step-filter'),
                        'number-to' => esc_html__('Number to', 'woocommerce-step-filter'),
                        'number-between' => esc_html__('Number between', 'woocommerce-step-filter')
                    ]
                ],
                'filter_type' => [
                    'label' => esc_html__('Filter type', 'woocommerce-step-filter'),
                    'key' => '_filter_type',
                    'type' => 'select',
                    'default' => 'select',
                    'values' => [
                        'attribute' => esc_html__('Attribute', 'woocommerce-step-filter'),
                        'price' => esc_html__('Price', 'woocommerce-step-filter'),
                        'tag' => esc_html__('Tag', 'woocommerce-step-filter'),
                        'category' => esc_html__('Category', 'woocommerce-step-filter'),
                        'manual' => esc_html__('Manual', 'woocommerce-step-filter'),
                        'meta' => esc_html__('Meta', 'woocommerce-step-filter')
                    ]
                ],
                'meta_key' => [
                    'label' => esc_html__('Meta key', 'woocommerce-step-filter'),
                    'key' => '_meta_key',
                    'type' => 'text',
                    'default' => esc_html__('', 'woocommerce-step-filter')
                ],
                'multi_choice_values' => [
                    'label' => esc_html__('Possible values', 'woocommerce-step-filter'),
                    'key' => '_multi_choice_possible_values',
                    'type' => 'multi-select',
                    'values' => [
                        0 => esc_html__('Undefined', 'woocommerce-step-filter'),
                        1 => esc_html__('Possible', 'woocommerce-step-filter'),
                        2 => esc_html__('Required', 'woocommerce-step-filter'),
                        -1 => esc_html__('Excluded', 'woocommerce-step-filter'),
                    ],
                    'default' => [0, 1, 2, -1]
                ],
                'multi_choice_undefined_value_text' => [
                    'label' => esc_html__('"Undefined" value text', 'woocommerce-step-filter'),
                    'key' => '_multi_choice_undefined_value_text',
                    'type' => 'text',
                    'default' => esc_html__('Undefined', 'woocommerce-step-filter')
                ],
                'multi_choice_possible_value_text' => [
                    'label' => esc_html__('"Possible" value text', 'woocommerce-step-filter'),
                    'key' => '_multi_choice_possible_value_text',
                    'type' => 'text',
                    'default' => esc_html__('Possible', 'woocommerce-step-filter')
                ],
                'multi_choice_required_value_text' => [
                    'label' => esc_html__('"Required" value text', 'woocommerce-step-filter'),
                    'key' => '_multi_choice_required_value_text',
                    'type' => 'text',
                    'default' => esc_html__('Required', 'woocommerce-step-filter')
                ],
                'multi_choice_excluded_value_text' => [
                    'label' => esc_html__('"Excluded" value text', 'woocommerce-step-filter'),
                    'key' => '_multi_choice_excluded_value_text',
                    'type' => 'text',
                    'default' => esc_html__('Excluded', 'woocommerce-step-filter')
                ],
                'from_value_text' => [
                    'label' => esc_html__('"From" value text', 'woocommerce-step-filter'),
                    'key' => '_from_value_text',
                    'type' => 'text',
                    'default' => esc_html__('From', 'woocommerce-step-filter')
                ],
                'to_value_text' => [
                    'label' => esc_html__('"To" value text', 'woocommerce-step-filter'),
                    'key' => '_to_value_text',
                    'type' => 'text',
                    'default' => esc_html__('To', 'woocommerce-step-filter')
                ],
                'numeric_unit' => [
                    'label' => esc_html__('"From" and "To" unit', 'woocommerce-step-filter'),
                    'key' => '_numeric_unit',
                    'type' => 'text',
                    'default' => ''
                ],
                'numeric_step' => [
                    'label' => esc_html__('Numeric value step', 'woocommerce-step-filter'),
                    'key' => '_numeric_step',
                    'type' => 'number',
                    'step' => 0.0001,
                    'min' => 0.0001,
                    'default' => 1
                ],
                'alignment' => [
                    'label' => esc_html__('Thumbnails alignment', 'woocommerce-step-filter'),
                    'key' => '_alignment',
                    'type' => 'select',
                    'default' => 'select',
                    'values' => [
                        'start' => esc_html__('Start', 'woocommerce-step-filter'),
                        'end' => esc_html__('End', 'woocommerce-step-filter'),
                        'center' => esc_html__('Center', 'woocommerce-step-filter'),
                        'between' => esc_html__('Between', 'woocommerce-step-filter'),
                        'around' => esc_html__('Around', 'woocommerce-step-filter'),
                        'evenly' => esc_html__('Evenly', 'woocommerce-step-filter')
                    ]
                ],
                'thumbnail_size' => [
                    'label' => esc_html__('Thumbnail size', 'woocommerce-step-filter'),
                    'key' => '_thumbnail_size',
                    'type' => 'text',
                    'default' => 'thumbnail',
                    'description' => esc_html__('For example: thumbnail, medium, large', 'woocommerce-step-filter'),
                ],
                'category_parent' => [
                    'label' => esc_html__('Category parent', 'woocommerce-step-filter'),
                    'key' => '_category_parent',
                    'type' => 'select',
                    'default' => 0,
                    'values' => []
                ],
                'include_sub_categories' => [
                    'label' => esc_html__('Include sub-categories', 'woocommerce-step-filter'),
                    'key' => '_include_sub_categories',
                    'type' => 'checkbox',
                    'default' => false
                ],
                'included_terms' => [
                    'label' => esc_html__('Included terms', 'woocommerce-step-filter'),
                    'key' => '_included_terms',
                    'type' => 'text',
                    'default' => '',
                    'description' => esc_html__(
                        'Define terms IDs with comma to use only them in the question',
                        'woocommerce-step-filter'
                    )
                ],
                'excluded_terms' => [
                    'label' => esc_html__('Excluded terms', 'woocommerce-step-filter'),
                    'key' => '_excluded_terms',
                    'type' => 'text',
                    'default' => '',
                    'description' => esc_html__(
                        'Define terms IDs with comma to not use them in the question',
                        'woocommerce-step-filter'
                    )
                ],
                'show_count' => [
                    'label' => esc_html__('Show count', 'woocommerce-step-filter'),
                    'key' => '_show_count',
                    'type' => 'checkbox',
                    'default' => false,
                    'description' => esc_html__('Show term products count', 'woocommerce-step-filter')
                ],
                'attribute' => [
                    'label' => esc_html__('Attribute', 'woocommerce-step-filter'),
                    'key' => '_attribute',
                    'type' => 'select',
                    'default' => 0,
                    'values' => []
                ],
                'action_on_click' => [
                    'label' => esc_html__('Action on click', 'woocommerce-step-filter'),
                    'key' => '_action_on_click',
                    'type' => 'select',
                    'default' => 'none',
                    'values' => [
                        'none' => esc_html__('None', 'woocommerce-step-filter'),
                        'submit' => esc_html__('Submit', 'woocommerce-step-filter'),
                        'submit-and-go-next' => esc_html__('Submit and go next', 'woocommerce-step-filter')
                    ]
                ],
                'relation' => [
                    'label' => esc_html__('Relation', 'woocommerce-step-filter'),
                    'key' => '_relation',
                    'type' => 'select',
                    'default' => 'or',
                    'values' => [
                        'or' => esc_html__('OR', 'woocommerce-step-filter'),
                        'and' => esc_html__('AND', 'woocommerce-step-filter')
                    ]
                ],
                'any_product_value' => [
                    'label' => esc_html__('Add "Doesn\'t matter" value', 'woocommerce-step-filter'),
                    'key' => '_any_product_value',
                    'type' => 'checkbox',
                    'default' => false
                ],
                'any_product_value_text' => [
                    'label' => esc_html__('"Doesn\'t matter" value text', 'woocommerce-step-filter'),
                    'key' => '_any_product_value_text',
                    'type' => 'text',
                    'default' => esc_html__('Doesn\'t matter', 'woocommerce-step-filter')
                ],
                'any_product_value_thumbnail' => [
                    'label' => esc_html__('"Doesn\'t matter" value thumbnail', 'woocommerce-step-filter'),
                    'key' => '_any_product_value_thumbnail',
                    'type' => 'thumbnail'
                ],
                'dependencies' => [
                    'label' => esc_html__('Dependencies', 'woocommerce-step-filter'),
                    'key' => '_dependencies',
                    'type' => 'data-table',
                    'values' => [
                        'question' => [
                            'label' => esc_html__('Question', 'woocommerce-step-filter'),
                            'key' => 'question',
                            'type' => 'select',
                            'values' => [],
                            'default' => ''
                        ],
                        'value' => [
                            'label' => esc_html__('Value', 'woocommerce-step-filter'),
                            'key' => 'value',
                            'type' => 'ajax-select',
                            'target-parent' => '[data-component="wcsf-data-table-item"]',
                            'target-selector' => '[data-key="question"] select',
                            'action' => 'wcsfGetQuestionValuesSelectOptions',
                            'default' => ''
                        ],
                        'operator' => [
                            'label' => esc_html__('Operator', 'woocommerce-step-filter'),
                            'key' => 'operator',
                            'type' => 'select',
                            'values' => [
                                '=' => esc_html__('Equal', 'woocommerce-step-filter'),
                                '!=' => esc_html__('Not equal', 'woocommerce-step-filter'),
                                '<' => esc_html__('Less', 'woocommerce-step-filter'),
                                '<=' => esc_html__('Less or equal', 'woocommerce-step-filter'),
                                '>' => esc_html__('More', 'woocommerce-step-filter'),
                                '>=' => esc_html__('More or equal', 'woocommerce-step-filter')
                            ],
                            'default' => '='
                        ],
                        'upper_value' => [
                            'label' => esc_html__('Upper', 'woocommerce-step-filter'),
                            'key' => 'upper_value',
                            'type' => 'checkbox',
                            'default' => false
                        ],
                        'relation' => [
                            'label' => esc_html__('Relation', 'woocommerce-step-filter'),
                            'key' => 'relation',
                            'type' => 'select',
                            'values' => [
                                'or' => 'OR',
                                'and' => 'AND'
                            ],
                            'default' => 'or'
                        ]
                    ],
                    'default' => [],
                    'description' =>
                        esc_html__(
                            'The question will be shown only in the case of the meeting of a dependency',
                            'woocommerce-step-filter'
                        )
                        . '<br>'
                        . esc_html__(
                            'Upper value means the biggest value for the range type',
                            'woocommerce-step-filter'
                        )
                ],
                'disable_unmet_values' => [
                    'label' => esc_html__('Disable unmet values', 'woocommerce-step-filter'),
                    'key' => '_disable_unmet_values',
                    'type' => 'checkbox',
                    'default' => false,
                    'description' => esc_html__(
                        'Values which will lead to zero results will be disabled',
                        'woocommerce-step-filter'
                    )
                ],
                'bonded_questions' => [
                    'label' => esc_html__('Bind with questions', 'woocommerce-step-filter'),
                    'key' => '_bonded_questions',
                    'type' => 'multi-select',
                    'default' => [],
                    'values' => self::getPostsIdsList(),
                    'description' => esc_html__(
                        'Question value will be passed to the bonded questions too',
                        'woocommerce-step-filter'
                    ),
                ],
                'manual_values' => [
                    'label' => esc_html__('Manual values', 'woocommerce-step-filter'),
                    'key' => '_manual_values',
                    'type' => 'data-table',
                    'default' => []
                ]
            ]
        );

        $name = 'SF Questions';
        $args = [
            'label' => $name,
            'labels' => [
                'name' => $name,
                'singular_name' => $name,
                'menu_name' => $name
            ],
            'description' => esc_html__(
                'This is where you can add new questions for step filter',
                'woocommerce-step-filter'
            ),
            'public' => false,
            'show_ui' => true,
            'map_meta_cap' => true,
            'publicly_queryable' => false,
            'exclude_from_search' => true,
            'show_in_menu' => current_user_can('manage_woocommerce') ? 'woocommerce' : true,
            'hierarchical' => false,
            'rewrite' => false,
            'query_var' => false,
            'supports' => [
                'title',
                'thumbnail',
                'editor'
            ],
            'show_in_nav_menus' => false,
            'show_in_admin_bar' => true
        ];

        register_post_type('wcsf_question', $args);
    }
    // </editor-fold>

    // <editor-fold desc="Admin">
    /** Admin styles and scripts enqueue */
    public function enqueueAdmin()
    {
        $path = WC_STEP_FILTER_DEBUG ? 'src' : 'assets';
        $suffix = WC_STEP_FILTER_DEBUG ? '' : '.min';

        wp_enqueue_script(
            'woocommerce-step-filter-question',
            WC_STEP_FILTER_PLUGIN_URL . "$path/admin/js/question$suffix.js",
            'jquery',
            WC_STEP_FILTER_PLUGIN_URL,
            true
        );
    }

    /**
     * Save meta values
     *
     * @param integer $postId
     */
    public function savePostAction($postId)
    {
        if ((defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) || !current_user_can('edit_page', $postId)) {
            return;
        }

        foreach (self::$settingsModel as $setting) {
            $value = '';

            if (isset($_POST[$setting['key']])) {
                $value = $_POST[$setting['key']];
            }

            update_post_meta($postId, $setting['key'], $value);
        }
    }

    /** Add meta-boxes */
    public function addMetaBoxes()
    {
        add_meta_box(
            'woocommerce-step-filter',
            esc_html__('Settings', 'woocommerce-step-filter'),
            [$this, 'metaBoxCallback'],
            'wcsf_question',
            'normal'
        );
    }

    /**
     * Question page meta-box content
     *
     * @param object $post
     */
    public function metaBoxCallback($post)
    {
        global $_wp_additional_image_sizes;

        $manualValues = self::getSetting($post->ID, 'manual_values');
        $attributes = wc_get_attribute_taxonomies();
        $settingsModel = self::$settingsModel;
        $settingsModel['bonded_questions']['values'] = self::getPostsIdsList(['exclude' => $post->ID]);
        $imageSizes = [];

        foreach ($_wp_additional_image_sizes as $imageSizeName => $imageSize) {
            $crop = $imageSize['crop'] ? ' crop' : '';
            $imageSizes[] = "$imageSizeName ($imageSize[width]x$imageSize[height]$crop)";
        }

        // unset settings model for custom output
        unset($settingsModel['manual_values']);

        // blend in required values
        $settingsModel['thumbnail_size']['description'] .= ', ' . implode(', ', $imageSizes);
        $settingsModel['dependencies']['values']['question']['values'] = array_replace(
            ['' => ''],
            self::getPostsIdsList(['exclude' => [$post->ID]])
        );

        $settingsModel['category_parent']['values'] = array_replace(
            [0 => ''],
            self::getCategoriesTree()
        );

        foreach ($attributes as $attribute) {
            $settingsModel['attribute']['values'][$attribute->attribute_id] = $attribute->attribute_label;
        }

        $attributeKeys = array_keys($settingsModel['attribute']['values']);
        $settingsModel['attribute']['default'] = reset($attributeKeys);
        ?>
        <div class="wcsf-question"
            data-component="wcsf-question"
            data-post-id="<?php echo esc_attr($post->ID); ?>"
            data-ajax-url="<?php echo admin_url('admin-ajax.php'); ?>">
            <table class="wcsf-settings-table form-table wcsf-data-table">
                <?php foreach ($settingsModel as $settingKey => $setting) { ?>
                    <tr valign="top" data-component="wcsf-question-setting" data-key="<?php
                    echo esc_attr($settingKey);
                    ?>">
                        <th scope="row" class="titledesc">
                            <label for="<?php echo esc_attr($setting['key']); ?>"><?php
                                echo wp_kses_post($setting['label']);
                                ?></label>
                        </th>
                        <td>
                            <?php
                            Core::settingFieldView(
                                $setting,
                                ['values' => [$setting['key'] => self::getSetting($post->ID, $settingKey)]]
                            );
                            ?>
                        </td>
                    </tr>
                <?php } ?>
                <tr class="wcsf-question-manual" data-component="wcsf-question-setting" data-key="manual">
                    <th><?php esc_html_e('Manual values', 'woocommerce-step-filter'); ?></th>
                    <td>
                        <button class="button-primary"
                            data-component="wcsf-question-add-value"><?php
                            esc_html_e('Add a value', 'woocommerce-step-filter');
                            ?></button><br><br>
                        <div class="wcsf-question-manual-values">
                            <table class="wp-list-table widefat striped">
                                <thead>
                                    <tr>
                                        <th><?php esc_html_e('Image', 'woocommerce-step-filter'); ?></th>
                                        <th><?php esc_html_e('Pre-checked', 'woocommerce-step-filter'); ?></th>
                                        <th><?php esc_html_e('Name', 'woocommerce-step-filter'); ?></th>
                                        <th><?php esc_html_e('Dependencies', 'woocommerce-step-filter'); ?></th>
                                        <th><?php esc_html_e('Source', 'woocommerce-step-filter'); ?></th>
                                        <th><?php esc_html_e('Remove', 'woocommerce-step-filter'); ?></th>
                                    </tr>
                                </thead>
                                <tbody data-component="wcsf-question-manual-values">
                                    <?php
                                    if (is_array($manualValues)) {
                                        foreach ($manualValues as $valueKey => $value) {
                                            $this->renderManualQuestionValue(
                                                array_merge(
                                                    (array) $value,
                                                    [
                                                        'postId' => $post->ID,
                                                        'index' => $valueKey
                                                    ]
                                                )
                                            );
                                        }
                                    } else {
                                        $this->renderManualQuestionValue(['postId' => $post->ID]);
                                    }
                                    ?>
                                </tbody>
                            </table>
                        </div>
                    </td>
                </tr>
            </table>
        </div>
        <?php
    }

    /** Return manual value html */
    public function addQuestionValueAjax()
    {
        $this->renderManualQuestionValue($_GET);

        die();
    }

    /**
     * Return table of manual values html
     *
     * @param array $args
     */
    public function renderManualQuestionValue($args = [])
    {
        $defaults = [
            'postId' => 0,
            'index' => 0,
            'name' => '',
            'thumbnail' => '',
            'pre-checked' => 0,
            'dependencies' => []
        ];

        $args = array_replace($defaults, $args);
        $dependenciesValuesModel = self::$settingsModel['dependencies']['values'];
        $dependenciesValuesModel['question']['values'] = array_replace(
            ['' => ''],
            self::getPostsIdsList(['exclude' => [$args['postId']]])
        );
        ?>
        <tr data-component="wcsf-question-manual-value" data-index="<?php echo esc_attr($args['index']); ?>">
            <td>
                <?php
                Core::settingFieldView(
                    [
                        'label' => esc_html__('Thumbnail', 'woocommerce-step-filter'),
                        'key' => esc_attr("_manual_values[$args[index]][thumbnail]"),
                        'type' => 'thumbnail',
                        'default' => ''
                    ],
                    ['values' => [esc_attr("_manual_values[$args[index]][thumbnail]") => $args['thumbnail']]]
                );
                ?>
            </td>
            <td>
                <input type="hidden"
                    name="_manual_values[<?php echo esc_attr($args['index']); ?>][pre-checked]"
                    value="0">
                <label>
                    <input type="checkbox"
                        name="_manual_values[<?php echo esc_attr($args['index']); ?>][pre-checked]"
                        class="regular-text wcsf-question-manual-values-item-pre-checked"
                        value="1"
                        <?php echo $args['pre-checked'] ? ' checked="checked"' : ''; ?>>
                    <span class="screen-reader-text"><?php
                        esc_html_e('Pre-checked', 'woocommerce-step-filter');
                        ?></span>
                </label>
            </td>
            <td>
                <input type="text"
                    name="_manual_values[<?php echo esc_attr($args['index']); ?>][name]"
                    class="regular-text wcsf-question-manual-values-item-name"
                    placeholder="<?php esc_attr_e('Some value here', 'woocommerce-step-filter'); ?>"
                    value="<?php echo esc_attr($args['name']); ?>">
            </td>
            <td>
                <a href="#wcsf-data-table-modal-dependencies-<?php echo esc_attr($args['index']); ?>"
                    class="button wcsf-data-table-item-open-modal"
                    data-component="wcsf-open-modal"><?php
                    esc_html_e('Set', 'woocommerce-step-filter');
                    ?></a>
                <div id="wcsf-data-table-modal-dependencies-<?php echo esc_attr($args['index']); ?>"
                    class="wcsf-modal"
                    data-component="wcsf-modal">
                    <div class="wcsf-modal-dialog">
                        <a href="#close"
                            title="<?php esc_html_e('Close', 'woocommerce-step-filter'); ?>"
                            class="wcsf-modal-close"
                            data-component="wcsf-modal-close">&times;</a>
                        <?php
                        Core::settingFieldView(
                            [
                                'label' => esc_html__('Dependencies', 'woocommerce-step-filter'),
                                'key' => esc_attr("_manual_values[$args[index]][dependencies]"),
                                'type' => 'data-table',
                                'values' => $dependenciesValuesModel,
                                'default' => [],
                                'description' =>
                                    esc_html__(
                                        'The value will be shown only in the case of the meeting of a dependency',
                                        'woocommerce-step-filter'
                                    )
                                    . '<br>'
                                    . esc_html__(
                                        'Upper value means the biggest value for the range type',
                                        'woocommerce-step-filter'
                                    )
                            ],
                            [
                                'values' => [
                                    esc_attr("_manual_values[$args[index]][dependencies]") => $args['dependencies']
                                ]
                            ]
                        );
                        ?>
                        <div class="wcsf-modal-dialog-footer">
                            <a href="#close" class="button button-primary" data-component="wcsf-modal-close"><?php
                                esc_html_e('Save', 'woocommerce-step-filter');
                                ?></a>
                        </div>
                    </div>
                </div>
            </td>
            <td>
                <a href="#wcsf-data-table-modal-source-<?php echo esc_attr($args['index']); ?>"
                    class="button-primary wcsf-data-table-item-open-modal"
                    data-component="wcsf-open-modal"><?php
                    esc_html_e('Set', 'woocommerce-step-filter');
                    ?></a>
                <div id="wcsf-data-table-modal-source-<?php echo esc_attr($args['index']); ?>"
                    class="wcsf-modal"
                    data-component="wcsf-modal">
                    <div class="wcsf-modal-dialog">
                        <a href="#close"
                            title="<?php esc_html_e('Close', 'woocommerce-step-filter'); ?>"
                            class="wcsf-modal-close"
                            data-component="wcsf-modal-close">&times;</a>
                        <ul>
                            <?php
                            $this->renderAnyProductField($args);
                            $this->renderCategorySelectFields($args);
                            $this->renderProductSelectFields($args);
                            ?>
                        </ul>
                        <div class="wcsf-modal-dialog-footer">
                            <a href="#close" class="button button-primary" data-component="wcsf-modal-close"><?php
                                esc_html_e('Save', 'woocommerce-step-filter');
                                ?></a>
                        </div>
                    </div>
                </div>
            </td>
            <td>
                <button class="button-secondary"
                    title="<?php esc_attr_e('Remove', 'woocommerce-step-filter'); ?>"
                    data-component="wcsf-question-manual-value-remove"
                    data-index="<?php echo esc_attr($args['index']); ?>">&times;</button>
            </td>
        </tr>
        <?php
    }

    /**
     * Get list of an option values
     *
     * @param integer|array $id
     *
     * @return array
     */
    public function getValuesSelectOptions($id)
    {
        $output = [];
        $filterType = self::getSetting($id, 'filter_type');
        $attributes = self::getSetting($id, 'attribute');
        $viewType = self::getSetting($id, 'view_type');
        $anyProductValue = self::getSetting($id, 'any_product_value');
        $manualValues = self::getSetting($id, 'manual_values');

        if ($filterType == 'manual') {
            foreach ($manualValues as $key => $value) {
                $output[$key] = $value['name'];
            }
        } elseif ($filterType == 'attribute') {
            $taxonomy = wc_attribute_taxonomy_name_by_id((int) $attributes);
            $terms = get_terms(['taxonomy' => $taxonomy]);

            foreach ($terms as $term) {
                $key = strpos($viewType, 'number') !== false ? $term->name : $term->term_id;
                $output[$key] = $term->name;
            }
        } elseif ($filterType == 'price') {
            static $prices;

            if (empty($prices)) {
                $prices = Core::getProductsPrices();
            }

            foreach ($prices as $price) {
                $output[$price] = $price;
            }
        } elseif (in_array($filterType, ['tag', 'category'])) {
            $taxonomy = $filterType == 'tag' ? 'product_tag' : 'product_cat';
            $terms = Core::getTermsTree(
                ['taxonomy' => $taxonomy],
                [
                    'flatten' => true,
                    'showLevelInName' => true
                ]
            );

            foreach ($terms as $term) {
                $output[$term->term_id] = $term->name;
            }
        }

        if (in_array($viewType, ['radio', 'select', 'image-radio']) && $anyProductValue) {
            $output[0] = esc_html__('Doesn\'t matter', 'woocommerce-step-filter');
        }

        asort($output, SORT_NUMERIC);

        return $output;
    }

    /** Get list of an option values via Ajax */
    public function getValuesSelectOptionsAjax()
    {
        $output = [];
        $values = $this->getValuesSelectOptions((int) $_GET['value']);

        foreach ($values as $key => $value) {
            $output[] = "<option value=\"$key\">$value</option>";
        }

        echo implode('', $output);

        exit;
    }

    /**
     * Return "any product" radio html
     *
     * @param array $args
     */
    public function renderAnyProductField($args = [])
    {
        $defaults = [
            'index' => 0,
            'filter-by' => 'category'
        ];

        $args = array_replace($defaults, $args);

        $isChecked = $args['filter-by'] == 'any';
        ?>
        <li class="wcsf-question-manual-value-filter<?php echo !$isChecked ? ' is-closed' : ''; ?>"
            data-component="wcsf-question-manual-value-filter"
            data-id="any">
            <label class="wcsf-question-manual-value-filter-label">
                <input type="radio"
                    data-component="wcsf-question-manual-value-filter-by"
                    data-id="any"
                    name="_manual_values[<?php echo esc_attr($args['index']); ?>][filter-by]"
                    value="any"<?php echo $isChecked ? ' checked="checked"' : ''; ?>>
                <?php esc_html_e('Any product', 'woocommerce-step-filter'); ?>
            </label>
        </li>
        <?php
    }

    /**
     * Return filter by category radio html
     *
     * @param array $args
     */
    public function renderCategorySelectFields($args = [])
    {
        $defaults = [
            'index' => 0,
            'items' => [],
            'type' => 'select',
            'filter-by' => 'category'
        ];

        $args = array_replace($defaults, $args);

        $isChecked = $args['filter-by'] == 'category';
        ?>
        <li class="wcsf-question-manual-value-filter<?php echo !$isChecked ? ' is-closed' : ''; ?>"
            data-component="wcsf-question-manual-value-filter"
            data-id="category">
            <label class="wcsf-question-manual-value-filter-label">
                <input type="radio"
                    data-component="wcsf-question-manual-value-filter-by"
                    data-id="category"
                    name="_manual_values[<?php echo esc_attr($args['index']); ?>][filter-by]"
                    value="category"<?php echo $isChecked ? ' checked="checked"' : ''; ?>>
                <?php esc_html_e('Filter by category', 'woocommerce-step-filter'); ?>
            </label>
            <?php $this->renderCategorySelectFieldsValues($args, 0, $isChecked); ?>
        </li>
        <?php
    }

    /**
     * Return list of categories checkbox html
     *
     * @param array $args
     * @param integer $parent
     * @param bool $isChecked
     *
     * @return string
     */
    public function renderCategorySelectFieldsValues($args, $parent, $isChecked)
    {
        $terms = get_terms([
            'taxonomy' => 'product_cat',
            'parent' => $parent,
            'hide_empty' => false
        ]);

        if (empty($terms) || is_wp_error($terms)) {
            return '';
        }
        ?>
        <table class="wp-list-table widefat striped wcsf-question-manual-value-filter-values">
            <?php foreach ($terms as $term) { ?>
                <tr>
                    <td>
                        <label>
                            <input type="checkbox"
                                name="_manual_values[<?php echo esc_attr($args['index']); ?>][items][]"
                                value="<?php echo esc_attr($term->term_id); ?>"<?php
                                echo $isChecked && in_array($term->term_id, $args['items']) ? ' checked' : '';
                            ?>><?php echo wp_kses_post($term->name); ?></label>
                        <?php $this->renderCategorySelectFieldsValues($args, $term->term_id, $isChecked); ?>
                    </td>
                </tr>
            <?php } ?>
        </table>
        <?php
        return '';
    }

    /**
     * Return list of categories in array
     *
     * @param integer $parent
     * @param integer $level
     *
     * @return array
     */
    public function getCategoriesTree($parent = 0, $level = 0)
    {
        $output = [];
        $terms = get_terms([
            'taxonomy' => 'product_cat',
            'parent' => $parent,
            'hide_empty' => false
        ]);

        if (empty($terms) || is_wp_error($terms)) {
            return [];
        }

        foreach ($terms as $term) {
            $output[$term->term_id] = trim(str_repeat('-', $level) . ' ' . wp_kses_post($term->name));
            $children = $this->getCategoriesTree($term->term_id, $level + 1);

            if (!empty($children)) {
                $output = array_replace($output, $children);
            }
        }

        return $output;
    }

    /**
     * Return filter by product radio html
     *
     * @param array $args
     */
    public function renderProductSelectFields($args = [])
    {
        $defaults = [
            'index' => 0,
            'items' => [],
            'type' => 'select',
            'filter-by' => 'category'
        ];

        $args = array_replace($defaults, $args);

        $isChecked = $args['filter-by'] == 'product';
        ?>
        <li class="wcsf-question-manual-value-filter<?php echo !$isChecked ? ' is-closed' : ''; ?>"
            data-component="wcsf-question-manual-value-filter"
            data-id="product">
            <label class="wcsf-question-manual-value-filter-label">
                <input type="radio"
                    data-component="wcsf-question-manual-value-filter-by"
                    data-id="product"
                    name="_manual_values[<?php echo esc_attr($args['index']); ?>][filter-by]"
                    value="product"<?php echo $isChecked ? ' checked="checked"' : ''; ?>>
                <?php esc_html_e('Select products manually', 'woocommerce-step-filter'); ?>
            </label>
            <?php
            $this->renderProductSelectFieldsProducts($args, false);
            $this->renderProductSelectFieldsCategories($args, 0);
            ?>
        </li>
        <?php
    }

    /**
     * Return list of products with categories checkboxes html
     *
     * @param array $args
     * @param integer $parent
     *
     * @return string
     */
    public function renderProductSelectFieldsCategories($args, $parent)
    {
        $terms = get_terms([
            'taxonomy' => 'product_cat',
            'parent' => $parent,
            'hide_empty' => false
        ]);

        if (empty($terms) || is_wp_error($terms)) {
            return '';
        }
        ?>
        <table class="wcsf-question-manual-value-filter-values wp-list-table widefat striped product">
            <?php foreach ($terms as $term) { ?>
                <tr class="wcsf-question-manual-value-filter-values-item">
                    <td class="wcsf-question-manual-value-filter-values-item-name">
                        <?php
                        echo wp_kses_post($term->name);

                        $this->renderProductSelectFieldsProducts($args, $term->term_id);
                        $this->renderProductSelectFieldsCategories($args, $term->term_id);
                        ?>
                    </td>
                </tr>
            <?php } ?>
        </table>
        <?php
        return '';
    }

    /**
     * Return list of products checkboxes html
     *
     * @param array $args
     * @param integer $parent
     *
     * @return string
     */
    public function renderProductSelectFieldsProducts($args, $parent)
    {
        $defaults = [
            'items' => []
        ];

        $args = array_replace($defaults, $args);

        $query = [
            'post_type' => 'product',
            'posts_per_page' => -1
        ];

        if ($parent !== false) {
            $query['tax_query'] = [
                [
                    'taxonomy' => 'product_cat',
                    'terms' => $parent,
                    'include_children' => false
                ]
            ];
        } else {
            $query['tax_query'] = [
                [
                    'taxonomy' => 'product_cat',
                    'terms' => get_terms(['taxonomy' => 'product_cat', 'fields' => 'ids']),
                    'operator' => 'NOT IN',
                ]
            ];
        }

        $products = get_posts($query);

        if (empty($products)) {
            return '' ;
        }
        ?>
        <table class="wcsf-question-manual-value-filter-values wp-list-table widefat striped">
            <?php foreach ($products as $product) { ?>
                <tr class="wcsf-question-manual-value-filter-values-item">
                    <td>
                        <label>
                            <?php
                            echo wp_get_attachment_image(get_post_thumbnail_id($product->ID), 'shop_thumbnail');
                            ?>
                            <input type="checkbox"
                                name="_manual_values[<?php echo esc_attr($args['index']); ?>][items][]"
                                value="<?php echo esc_attr($product->ID); ?>"<?php
                                echo in_array($product->ID, $args['items']) ? ' checked' : '';
                            ?>>
                            <?php echo wp_kses_post($product->post_title); ?>
                        </label>
                    </td>
                </tr>
            <?php } ?>
        </table>
        <?php
        return '';
    }
    // </editor-fold>

    // <editor-fold desc="Settings">
    /**
     * Get one of post settings
     *
     * @param integer $id
     * @param string $setting
     *
     * @return string|boolean|array|float
     */
    public static function getSetting($id, $setting)
    {
        static $questionSettingsCache;

        if (isset($questionSettingsCache[$id], $questionSettingsCache[$id][$setting])) {
            return $questionSettingsCache[$id][$setting];
        }

        $value = isset(self::$settingsModel[$setting]['key'])
            ? get_post_meta($id, self::$settingsModel[$setting]['key'], true)
            : null;

        if ($value == null && isset(self::$settingsModel[$setting]['default'])) {
            $value = self::$settingsModel[$setting]['default'];
        }

        switch (self::$settingsModel[$setting]['type']) {
            case 'checkbox':
                $value = filter_var($value, FILTER_VALIDATE_BOOLEAN);
                break;

            case 'number':
                $value = (float) $value;
                break;

            case 'array':
                $value = (array) $value;
                break;
        }

        if (!isset($questionSettingsCache[$id])) {
            $questionSettingsCache[$id] = [];
        }

        $questionSettingsCache[$id][$setting] = $value;

        return $value;
    }
    // </editor-fold>

    // <editor-fold desc="Get">
    /**
     * Return question data object
     *
     * @param object $question - WP query post
     *
     * @return object
     */
    public static function getData($question)
    {
        $viewType = self::getSetting($question->ID, 'view_type');
        $multiChoiceValues = [];

        if (strpos($viewType, 'multi-choice') !== false) {
            $multiChoiceValuesIncluded = (array) self::getSetting($question->ID, 'multi_choice_values');
            $multiChoiceValues = [
                0 => [
                    'label' => self::getSetting($question->ID, 'multi_choice_undefined_value_text'),
                    'key' => 'undefined',
                    'value' => 0
                ],
                1 => [
                    'label' => self::getSetting($question->ID, 'multi_choice_possible_value_text'),
                    'key' => 'possible',
                    'value' => 1
                ],
                2 => [
                    'label' => self::getSetting($question->ID, 'multi_choice_required_value_text'),
                    'key' => 'required',
                    'value' => 2
                ],
                -1 => [
                    'label' => self::getSetting($question->ID, 'multi_choice_excluded_value_text'),
                    'key' => 'excluded',
                    'value' => -1
                ]
            ];

            foreach ($multiChoiceValues as $key => $multiChoiceValue) {
                if (in_array($multiChoiceValue['value'], $multiChoiceValuesIncluded)) {
                    continue;
                }

                unset($multiChoiceValues[$key]);
            }

            $multiChoiceValues = array_replace(array_flip($multiChoiceValuesIncluded), $multiChoiceValues);
        }

        $output = (object) [
            'id' => (int) $question->ID,
            'name' => $question->post_title,
            'description' => $question->post_content,
            'viewType' => $viewType,
            'filterType' => self::getSetting($question->ID, 'filter_type'),
            'attribute' => self::getSetting($question->ID, 'attribute'),
            'manualValues' => self::getSetting($question->ID, 'manual_values'),
            'thumbnailId' => get_post_thumbnail_id($question->ID),
            'thumbnailSize' => self::getSetting($question->ID, 'thumbnail_size'),
            'alignment' => self::getSetting($question->ID, 'alignment'),
            'relation' => self::getSetting($question->ID, 'relation'),
            'metaKey' => self::getSetting($question->ID, 'meta_key'),
            'multiChoiceValues' => array_values($multiChoiceValues)
        ];

        return apply_filters('wcsf_question_data', $output, $question);
    }

    /**
     * Return a question by id
     *
     * @param integer $id
     *
     * @return object
     */
    public static function get($id)
    {
        return apply_filters('wcsf_question', self::getData(get_post($id)), $id);
    }

    /**
     * Return array of questions
     *
     * @param array $ids
     *
     * @return array
     */
    public static function getList($ids)
    {
        global $wpdb;

        // using of an SQL query avoids cross wp_queries problems
        $output = [];
        $questionsIdsString = esc_sql(implode(', ', $ids));
        $queryString = "SELECT $wpdb->posts.* 
            FROM $wpdb->posts
            WHERE $wpdb->posts.post_status = 'publish' 
            AND $wpdb->posts.post_type = 'wcsf_question'
            AND $wpdb->posts.ID IN ($questionsIdsString)
            ORDER BY FIELD(ID, $questionsIdsString) ASC";

        $questions = $wpdb->get_results($queryString, OBJECT);

        foreach ($questions as $question) {
            $output[$question->ID] = self::getData($question);
        }

        return apply_filters('wcsf_questions_list', $output, $ids);
    }

    /**
     * Returns array of ids and titles of posts
     *
     * @param array $args
     *
     * @return array
     */
    public static function getPostsIdsList($args = [])
    {
        $defaults = [
            'post_type' => 'wcsf_question',
            'numberposts' => -1,
            'post_status' => 'publish'
        ];

        $args = array_replace($defaults, $args);
        $output = [];
        $posts = get_posts($args);

        foreach ($posts as $post) {
            $output[$post->ID] = $post->post_title;
        }

        return $output;
    }
    // </editor-fold>
}
