/* WooCommerce Step Filter Ajax Select
 * Original author: troll_winner@mail.ru
 * Further changes, comments: troll_winner@mail.ru
 */

(function (root, factory) {
    'use strict';

    if (typeof define === 'function' && define.amd) {
        define(['jquery'], factory);
    } else if (typeof exports === 'object'
        && typeof module !== 'undefined'
        && typeof require === 'function'
    ) {
        module.exports = factory(require('jquery'));
    } else {
        factory(root.jQuery);
    }
})(this, function ($) {
    'use strict';

    const pluginName = 'wcsfAjaxSelect';
    const defaults = {};

    const Plugin = function (element, options) {
        this.element = element;
        this.options = $.extend({}, defaults, options);

        return this.init();
    };

    // on plugin init
    Plugin.prototype.init = function () {
        this.$element = $(this.element);
        this.$target = false;
        this.ajaxUrl = this.$element.data('ajax-url');

        if (this.$element.data('target-parent')) {
            this.$target = this.$element.parents(this.$element.data('target-parent'));
        }

        if (this.$element.data('target-selector')) {
            if (this.$target) {
                this.$target = this.$target.find(this.$element.data('target-selector'));
            } else {
                this.$target = $(this.$element.data('target-selector'));
            }
        }

        // set items by default
        if (this.$element.data('value') !== '') {
            this.getItems().done(() => this.$element.val(this.$element.data('value')));
        }

        this.$target.on('change', () => this.getItems());

        this.$element.data('wcsf-ajax-select', this);

        return this;
    };

    // get items list via ajax
    Plugin.prototype.getItems = function () {
        const data = {
            action: this.$element.data('action'),
            value: this.$target.val()
        };

        return $.get(
            this.ajaxUrl,
            data,
            (response) => {
                this.$element.html(response);

                $(document).trigger('itemsUpdated.ajaxSelect.wcsf');
            }
        );
    };

    $.fn[pluginName] = function (options) {
        return this.each(function () {
            if (!$.data(this, 'plugin_' + pluginName)) {
                $.data(this, 'plugin_' + pluginName, new Plugin(this, options));
            }
        });
    };

    const init = () => $('[data-component="wcsf-ajax-select"]').each(function () {
        return $(this).wcsfAjaxSelect();
    });

    $(document)
        .ready(() => init())
        .on('ajaxComplete.wcsf init.ajaxSelect.wcsf', () => init());
});
