/* WooCommerce Step Filter DataTable
 * Original author: troll_winner@mail.ru
 * Further changes, comments: troll_winner@mail.ru
 */

(function (root, factory) {
    'use strict';

    if (typeof define === 'function' && define.amd) {
        define(['jquery'], factory);
    } else if (typeof exports === 'object'
        && typeof module !== 'undefined'
        && typeof require === 'function'
    ) {
        module.exports = factory(require('jquery'));
    } else {
        factory(root.jQuery);
    }
})(this, function ($) {
    'use strict';

    const pluginName = 'wcsfDataTable';
    const defaults = {};

    const Plugin = function (element, options) {
        this.element = element;
        this.options = $.extend({}, defaults, options);

        return this.init();
    };

    // on plugin init
    Plugin.prototype.init = function () {
        this.$element = $(this.element);

        if ($.fn.sortable) {
            // init sortable
            this.$element.sortable({
                items: '[data-component="wcsf-data-table-item"]',
                update: () => this.recalculation()
            });
        }

        this.$element.data('wcsf-data-table', this);

        return this;
    };

    // add the new element in the table
    Plugin.prototype.addItem = function ($insertAfterItem) {
        $insertAfterItem = $insertAfterItem.length !== 0
            ? $insertAfterItem
            : this.$element.find('[data-component="wcsf-data-table-item"]:last');

        let $template = this.$element.find('[data-component="wcsf-data-table-item-template"]');

        if ($template.length === 0) {
            $template = $insertAfterItem;
        }

        // make the new element from default template
        $template
            .clone()
            .insertAfter($insertAfterItem)
            .attr('data-component', 'wcsf-data-table-item')
            .find(':input')
            .each(function () {
                const $this = $(this);

                // make real attributes from placeholders
                $.each(this.attributes, (i, attr) => {
                    const name = attr.name;
                    const value = attr.value;

                    if (name.indexOf('data-make') === -1) {
                        return this;
                    }

                    return $this.attr(name.replace('data-make-', ''), value);
                });
            });

        this.recalculation();

        $(document).trigger('itemAdded.dataTable.wcsf');

        return this;
    };

    // remove element from the table
    Plugin.prototype.removeItem = function ($item) {
        if ($item.is(':only-child')) {
            // clear values of the last item
            $item.find(':input').val('');
        } else {
            // remove the non-last item
            $item.remove();
        }

        this.recalculation();

        $(document).trigger('itemRemoved.dataTable.wcsf');

        return this;
    };

    // reCalculate the input names indexes
    Plugin.prototype.recalculation = function () {
        return this.$element
            .find('[data-component="wcsf-data-table-item"]')
            .each(function (index) {
                return $(this)
                    .find(':input')
                    .each(function () {
                        const $input = $(this);
                        const name = $input.attr('name');

                        if (!name) {
                            return this;
                        }

                        const number = index.toString().split('').reverse().join('');

                        // replace the first array key from the end
                        return $input.attr(
                            'name',
                            name
                                .split('')
                                .reverse()
                                .join('')
                                .replace(/]\d+\[/, `]${number}[`)
                                .split('')
                                .reverse()
                                .join('')
                        );
                    });
            });
    };

    $.fn[pluginName] = function (options) {
        return this.each(function () {
            if (!$.data(this, 'plugin_' + pluginName)) {
                $.data(this, 'plugin_' + pluginName,
                    new Plugin(this, options));
            }
        });
    };

    const init = () => $('[data-component="wcsf-data-table"]').each(function () {
        return $(this).wcsfDataTable();
    });

    $(document)
        .ready(() => init())
        .on('ajaxComplete.wcsf init.dataTable.wcsf', () => init())

        // add the form item
        .on('click', '[data-component="wcsf-data-table-item-add"]', function () {
            const $button = $(this);
            const $wcsfDataTable = $button.closest('[data-component="wcsf-data-table"]');

            if ($wcsfDataTable.data('wcsf-data-table')) {
                return $wcsfDataTable
                    .data('wcsf-data-table')
                    .addItem($button.closest('[data-component="wcsf-data-table-item"]'));
            }

            return this;
        })

        // remove the form item
        .on('click', '[data-component="wcsf-data-table-item-remove"]', function () {
            const $button = $(this);
            const $wcsfDataTable = $button.closest('[data-component="wcsf-data-table"]');

            if ($wcsfDataTable.data('wcsf-data-table')) {
                return $wcsfDataTable
                    .data('wcsf-data-table')
                    .removeItem($button.closest('[data-component="wcsf-data-table-item"]'));
            }

            return this;
        });
});
