(function (root, factory) {
    'use strict';

    if (typeof define === 'function' && define.amd) {
        define(['jquery'], factory);
    } else if (typeof exports === 'object'
        && typeof module !== 'undefined'
        && typeof require === 'function'
    ) {
        module.exports = factory(require('jquery'));
    } else {
        factory(root.jQuery);
    }
})(this, function ($) {
    'use strict';

    $(document)
        // reLaunch thumbnail field
        .on('addQuestionValue.wcStepFilterQuestion', () => $(document).trigger('init.wcStepFilterThumbnail'))

        // reLaunch ajax select elements
        .on('itemAdded.dataTable.wcsf', () => $(document).trigger('init.ajaxSelect.wcsf'))

        // open modal via JS too to be sure everything is alright (some 3rd code makes it wrong)
        .on('click', '[data-component~="wcsf-open-modal"]', function (event) {
            event.preventDefault();

            const href = $(this).attr('href');

            $(href).addClass('is-opened');

            if (history.pushState) {
                history.pushState(null, null, href);
            } else {
                location.hash = href;
            }
        })

        // close modal
        .on('click', '[data-component~="wcsf-modal-close"]', function (event) {
            event.preventDefault();

            $(this).closest('[data-component~="wcsf-modal"]').removeClass('is-opened');

            if (history.pushState) {
                history.pushState(null, null, '#');
            } else {
                location.hash = '#';
            }
        })

        // toggle settings group
        .on('click', '[data-component~="wcsf-settings-group-toggle"]', function (event) {
            event.preventDefault();

            const $element = $(this);
            const $content = $('[data-component="wcsf-settings-group-content"]');

            return $content.removeClass('is-visible')
                .filter(`[data-id=${$element.data('id')}]`)
                .toggleClass('is-visible');
        })

        // inquiries pagination
        .on('click', '[data-component~="wcsf-inquiry-list-pagination-item-link"]', function (event) {
            event.preventDefault();

            const $element = $(this);
            const $table = $element.closest('[data-component~="wcsf-inquiry-list-table"]');

            $table.addClass('is-loading');

            return $.get(
                $table.data('ajax-url'),
                {
                    action: 'WCSFGetInquiriesListTable',
                    filterId: $table.data('filter-id'),
                    total: $table.data('total'),
                    page: $element.data('page')
                },
                (response) => $table.replaceWith(response)
            );
        })

        // purge inquiries
        .on('click', '[data-component~="wcsf-inquiries-purge"]', function (event) {
            if (!confirm('Confirm')) {
                event.preventDefault();
            }
        });
});
