(function (root, factory) {
    'use strict';

    if (typeof define === 'function' && define.amd) {
        define(['jquery'], factory);
    } else if (typeof exports === 'object'
        && typeof module !== 'undefined'
        && typeof require === 'function'
    ) {
        module.exports = factory(require('jquery'));
    } else {
        factory(root.jQuery);
    }
})(this, function ($) {
    'use strict';

    let hideTooltipTimeout = null;
    const $document = $(document);
    const init = () => {
        $('[data-component~="wcsf"]').each(function () {
            const $element = $(this);

            return $element.wcsf($element.data('options') || {});
        });
    };

    function hideTooltip($tooltip) {
        if ($tooltip.length === 0) {
            return null;
        }

        return $tooltip.fadeOut(200, () => {
            $tooltip.remove();
        });
    }

    function showTooltip($element, text) {
        $('[data-component~="wcsf-tooltip"]').remove();
        clearTimeout(hideTooltipTimeout);

        const $tooltip = $(`<span class="wcsf-tooltip" data-component="wcsf-tooltip" data-text="${text}"></span>`);

        $element.before($tooltip);

        hideTooltipTimeout = setTimeout(
            () => hideTooltip($tooltip),
            2500
        );
    }

    $document
        // init
        .ready(() => init())
        .on('init.wcsf', () => init())

        // reset click
        .on('click.reset.wcsf', '[data-component~="wcsf-reset"]', function (event) {
            const $element = $(this);
            const $wcsf = $element.closest('[data-component~="wcsf"]');

            if ($wcsf.data('wcsf')) {
                event.preventDefault();

                return $wcsf.data('wcsf').reset();
            }

            return this;
        })

        // value item remove click
        .on('click.remove.value.wcsf', '[data-component~="wcsf-value-item-remove"]', function (event) {
            const $element = $(this);
            const $wcsf = $element.closest('[data-component~="wcsf"]');

            if ($wcsf.data('wcsf')) {
                event.preventDefault();

                return $wcsf.data('wcsf').reset({
                    stepId: $element.data('id')
                });
            }

            return this;
        })

        // nav click
        .on('click.nav.wcsf', '[data-component~="wcsf-nav-item"]', (event) => event.preventDefault())

        // submit and go to click
        .on('click.submitAndGoTo.wcsf', '[data-component~="wcsf-submit-and-go-to"]', function (event) {
            const $element = $(this);
            const $wcsf = $element.closest('[data-component~="wcsf"]');

            if ($wcsf.data('wcsf')) {
                if ($element.prop('nodeName') === 'BUTTON') {
                    event.preventDefault();
                }

                return $wcsf.data('wcsf').submitAndGoTo({stepId: $element.data('id')});
            }

            return this;
        })

        // submit and go next click
        .on('click.submitAndGoNext.wcsf', '[data-component~="wcsf-submit-and-go-next"]', function (event) {
            const $element = $(this);
            const $wcsf = $element.closest('[data-component~="wcsf"]');

            if ($wcsf.data('wcsf')) {
                if ($element.prop('tagName') === 'BUTTON') {
                    event.preventDefault();
                }

                return $wcsf.data('wcsf').submitAndGoNext({stepId: $element.data('id')});
            }

            return this;
        })

        // submit click
        .on('click.submit.wcsf', '[data-component~="wcsf-submit"]', function (event) {
            const $element = $(this);
            const $wcsf = $element.closest('[data-component~="wcsf"]');

            if ($wcsf.data('wcsf')) {
                if ($element.prop('tagName') === 'BUTTON') {
                    event.preventDefault();
                }

                return $wcsf.data('wcsf').submit({stepId: $element.data('id')});
            }

            return this;
        })

        // get step click
        .on('click.goTo.wcsf', '[data-component~="wcsf-go-to"]', function (event) {
            const $element = $(this);
            const $wcsf = $element.closest('[data-component~="wcsf"]');

            if ($wcsf.data('wcsf')) {
                event.preventDefault();

                return $wcsf.data('wcsf').goTo({
                    stepId: $element.data('id'),
                    page: $element.data('page') ? $element.data('page') : 1
                });
            }

            return this;
        })

        // hide tooltips on click
        .on('click.tooltips.wcsf', '[data-component~="wcsf"]', () => hideTooltip($('[data-component~="wcsf-tooltip"]')))

        // get preliminary results count
        .on(
            'change.getPreliminaryResults.wcsf changed.getPreliminaryResults.wcsf',
            '[data-component~="wcsf-get-preliminary-results-count"]',
            function () {
                const $element = $(this);
                const $wcsf = $element.closest('[data-component~="wcsf"]');

                if ($wcsf.data('wcsf')) {
                    hideTooltip($('[data-component~="wcsf-tooltip"]'));

                    return $wcsf.data('wcsf').getPreliminaryResultsCount($element);
                }

                return this;
            }
        )

        // products order change
        .on('change.order.wcsf', '[data-component~="wcsf-ordering-select"]', function (event) {
            const $element = $(this);
            const $wcsf = $element.closest('[data-component~="wcsf"]');

            if ($wcsf.data('wcsf')) {
                event.preventDefault();

                return $wcsf.data('wcsf').goTo({
                    stepId: $element.data('id'),
                    orderBy: $element.val(),
                    page: 1
                });
            }

            return this;
        })

        // WCSF ajax complete
        .on('ajaxCompleted.wcsf', () => {
            // noUi slider init
            $document.trigger('init.nouislider');
        })

        // get preliminary results count
        .on('getPreliminaryResultsCount.wcsf', (event, instance, data, $element) =>
            showTooltip($element, data.message));
});
