<?php
if (!defined('ABSPATH')) {
    exit();
}

use \WCStepFilter\Core;
use \WCStepFilter\Filter;

$id = isset($id) ? $id : false;

if (!$id) {
    throw new Exception('Empty id');
}

$defaults = [
    'id' => $id,
    'step' => null,
    'nextStepId' => null,
    'prevStepId' => null,
    'value' => Filter::getValue($id),
    'mode' => Filter::getSetting($id, 'mode'),
    'enableBackButton' => Filter::getSetting($id, 'enable_back_button'),
    'showNextButton' => Filter::getSetting($id, 'show_next_button'),
    'enableResetButton' => Filter::getSetting($id, 'enable_reset_button'),
    'showResultsButton' => Filter::getSetting($id, 'show_results_button'),
    'enableSkipButton' => Filter::getSetting($id, 'enable_skip_button')
];

$arguments = isset($arguments) ? array_replace_recursive($defaults, $arguments) : $defaults;

$isSingleStepMode = $arguments['mode'] == 'single-step';
$isDescriptionTab = $arguments['step']->id == 'description';
$isResultsTab = $arguments['step']->id == 'results';

switch ($arguments['showResultsButton']) {
    case 'always':
        $showResults = true;
        break;

    case 'never':
        $showResults = false;
        break;

    case 'on-answer':
        $showResults = !empty($arguments['value']);
        break;

    case 'last-step':
        $showResults = !$arguments['nextStepId'];
        break;
}

echo '<div class="wcsf-controls">';

if ($arguments['enableResetButton'] && (!empty($arguments['value']) || $arguments['prevStepId'])) {
    Core::getTemplatePart('controls/reset', $arguments);
}

if (!$isSingleStepMode) {
    if ($arguments['enableSkipButton'] && $arguments['nextStepId'] && !$isDescriptionTab) {
        Core::getTemplatePart('controls/skip', $arguments);
    }

    if ($arguments['enableBackButton'] && $arguments['prevStepId']) {
        Core::getTemplatePart('controls/back', $arguments);
    }

    if (($arguments['showNextButton'] == 'until-last-step' && $arguments['nextStepId'])
        || $arguments['showNextButton'] == 'always'
    ) {
        Core::getTemplatePart('controls/next', $arguments);
    }
}

if ($showResults) {
    Core::getTemplatePart('controls/results', $arguments);
}

echo '</div>';
