<?php
if (!defined('ABSPATH')) {
    exit();
}

use \WCStepFilter\Core;
use \WCStepFilter\Filter;

$id = isset($id) ? $id : false;

if (!$id) {
    throw new Exception('Empty id');
}

$defaults = [
    'id' => $id,
    'step' => null,
    'fields' => [],
    'navView' => Filter::getSetting($id, 'nav_view'),
    'mode' => Filter::getSetting($id, 'mode'),
    'steps' => []
];

$arguments = isset($arguments) ? array_replace_recursive($defaults, $arguments) : $defaults;

if (!in_array($arguments['mode'], ['single-step', 'sequence']) && $arguments['step']->id == 'results') {
    // show products on the results step not for one-page views and exit
    Core::getTemplatePart('products/index', $arguments);

    return;
}
?>
<form class="wcsf-form"
    id="wcsf-form-<?php echo esc_attr($arguments['id']); ?>"
    action="#"
    method="POST"
    data-component="wcsf-form"
    data-step-id="<?php echo esc_attr($arguments['step']->id); ?>">
    <?php
    if (in_array($arguments['mode'], ['free-walk', 'step-by-step'])) {
        Core::getTemplatePart('nav/' . $arguments['navView'], $arguments);
    }

    Core::getTemplatePart('value', $arguments);

    if ($arguments['mode'] == 'single-step') {
        foreach ($arguments['steps'] as $step) {
            Filter::setActiveStep($arguments['id'], $step->id);

            $arguments['step'] = $step;
            $arguments['fields'] = Filter::getStepFields($id, $step->id);

            if ($step->id != 'results') {
                Core::getTemplatePart('form/content', $arguments);
            }
        }
    } elseif ($arguments['mode'] == 'sequence') {
        $activeStep = Filter::getActiveStep($arguments['id']);

        foreach ($arguments['steps'] as $step) {
            Filter::setActiveStep($arguments['id'], $step->id);

            $arguments['step'] = $step;
            $arguments['fields'] = Filter::getStepFields($id, $step->id);

            if ($step->id != 'results') {
                Core::getTemplatePart('form/content', $arguments);
            }

            if ($activeStep->id == $step->id) {
                break;
            }
        }
    } else {
        Core::getTemplatePart('form/content', $arguments);
    }

    Core::getTemplatePart('controls/index', $arguments);
    ?>
    <input type="hidden" name="id" value="<?php echo esc_attr($arguments['id']); ?>">
    <input type="hidden" name="wcsf" value="<?php echo esc_attr($arguments['id']); ?>">
</form>
<?php
if (in_array($arguments['mode'], ['single-step', 'sequence']) && $step->id == 'results') {
    Core::getTemplatePart('products/index', $arguments);
}
?>
