<?php
if (!defined('ABSPATH')) {
    exit();
}

use \WCStepFilter\Core;
use \WCStepFilter\Filter;

$id = isset($id) ? $id : false;
$page = isset($page) ? $page : max(1, get_query_var('paged'));
$orderBy = isset($orderBy) ? $orderBy : 'menu_order';

if (!$id) {
    throw new Exception('Empty id');
}

$defaults = [
    'id' => $id,
    'navView' => Filter::getSetting($id, 'nav_view'),
    'mode' => Filter::getSetting($id, 'mode'),
    'productsQueryArgs' => Filter::getProductsQueryArgs($id, ['page' => $page], $orderBy)
];

$arguments = isset($arguments) ? array_replace_recursive($defaults, $arguments) : $defaults;

$productsQuery = new WP_Query($arguments['productsQueryArgs']);

do_action('wcsf_products_query', $id, $arguments, $productsQuery);

// change max products count
if ($arguments['productsQueryArgs']['maxResultsNumber']) {
    $productsQuery->post_count = min($arguments['productsQueryArgs']['maxResultsNumber'], $productsQuery->post_count);
    $productsQuery->found_posts = min($arguments['productsQueryArgs']['maxResultsNumber'], $productsQuery->found_posts);
}

$arguments = array_replace(['productsQuery' => $productsQuery], $arguments);

if (in_array($arguments['mode'], ['free-walk', 'step-by-step'])) {
    Core::getTemplatePart('nav/' . $arguments['navView'], $arguments);
    Core::getTemplatePart('value', $arguments);
}

echo '<div class="wcsf-products woocommerce">';

if ($productsQuery->have_posts()) {
    echo '<div class="wcsf-products-controls clearfix">';

    Core::getTemplatePart('products/order-by', $arguments);
    Core::getTemplatePart('products/result-count', $arguments);
    Core::getTemplatePart('products/pagination', $arguments);

    echo '</div>';

    woocommerce_product_loop_start();

    while ($productsQuery->have_posts()) {
        $productsQuery->the_post();

        do_action('woocommerce_shop_loop');
        wc_get_template_part('content', 'product');
    }

    woocommerce_product_loop_end();

    echo '<div class="wcsf-products-controls clearfix">';

    Core::getTemplatePart('products/order-by', $arguments);
    Core::getTemplatePart('products/result-count', $arguments);
    Core::getTemplatePart('products/pagination', $arguments);

    echo '</div>';
} else {
    /**
     * Hook: woocommerce_no_products_found.
     *
     * @hooked wc_no_products_found - 10
     */
    do_action('woocommerce_no_products_found');
}

Core::getTemplatePart('controls/index', $arguments);

echo '</div>';

wp_reset_postdata();
wp_reset_query();
